@@ -13,6 +13,7 @@ import math
 import numpy as np
 
 from .. import units as u
+from ..utils import deprecated
 
 __all__ = ['Distance', 'CartesianPoints', 'cartesian_to_spherical',
            'spherical_to_cartesian']
@@ -27,40 +28,54 @@ class Distance(u.Quantity):
 
     This can be initialized in one of four ways:
 
-    * A distance `value` (array or float) and a `unit`
-    * A `~astropy.units.quantity.Quantity` object
+    * A distance ``value`` (array or float) and a ``unit``
+    * A `~astropy.units.Quantity` object
     * A redshift and (optionally) a cosmology.
     * Providing a distance modulus
 
     Parameters
     ----------
-    value : scalar or `~astropy.units.quantity.Quantity`
-        The value of this distance
-    unit : `~astropy.units.core.UnitBase`
-        The units for this distance, *if* `value` is not a `Quantity`.
-        Must have dimensions of distance.
+    value : scalar or `~astropy.units.Quantity`.
+        The value of this distance.
+    unit : `~astropy.units.UnitBase`
+        The units for this distance, *if* ``value`` is not a
+        `~astropy.units.Quantity`. Must have dimensions of distance.
     z : float
         A redshift for this distance.  It will be converted to a distance
         by computing the luminosity distance for this redshift given the
-        cosmology specified by `cosmology`. Must be given as a keyword argument.
-    cosmology : `~astropy.cosmology.Cosmology` or None
-        A cosmology that will be used to compute the distance from `z`.
-        If None, the current cosmology will be used (see
+        cosmology specified by ``cosmology``. Must be given as a keyword
+        argument.
+    cosmology : ``Cosmology`` or `None`
+        A cosmology that will be used to compute the distance from ``z``.
+        If `None`, the current cosmology will be used (see
         `astropy.cosmology` for details).
     distmod : float or `~astropy.units.Quantity`
-        The distance modulus for this distance.
-    dtype : ~numpy.dtype, optional
-        See `~astropy.units.Quantity`. Must be given as a keyword argument.
+        The distance modulus for this distance. Note that if ``unit`` is not
+        provided, a guess will be made at the unit between AU, pc, kpc, and Mpc.
+    dtype : `~numpy.dtype`, optional
+        See `~astropy.units.Quantity`.
     copy : bool, optional
-        See `~astropy.units.Quantity`. Must be given as a keyword argument.
+        See `~astropy.units.Quantity`.
+    order : {'C', 'F', 'A'}, optional
+        See `~astropy.units.Quantity`.
+    subok : bool, optional
+        See `~astropy.units.Quantity`.
+    ndmin : int, optional
+        See `~astropy.units.Quantity`.
+    allow_negative : bool, optional
+        Whether to allow negative distances (which are possible is some
+        cosmologies).  Default: ``False``.
 
     Raises
     ------
-    astropy.units.core.UnitsError
-        If the `unit` is not a distance.
+    `~astropy.units.UnitsError`
+        If the ``unit`` is not a distance.
     ValueError
-        If `z` is provided with a `unit` or `cosmology` is provided when `z` is
-        *not* given, or `value` is given as well as `z`
+        If value specified is less than 0 and ``allow_negative=False``.
+
+        If ``z`` is provided with a ``unit`` or ``cosmology`` is provided
+        when ``z`` is *not* given, or ``value`` is given as well as ``z``.
+
 
     Examples
     --------
@@ -74,98 +89,80 @@ class Distance(u.Quantity):
     >>> d4 = Distance(z=0.23)
     >>> d5 = Distance(z=0.23, cosmology=WMAP5)
     >>> d6 = Distance(distmod=24.47)
+    >>> d7 = Distance(Distance(10 * u.Mpc))
     """
 
     _include_easy_conversion_members = True
 
     def __new__(cls, value=None, unit=None, z=None, cosmology=None,
-                distmod=None, dtype=None, copy=True):
-        from ..cosmology import get_current
+                distmod=None, dtype=None, copy=True, order=None,
+                subok=False, ndmin=0, allow_negative=False):
 
-        if isinstance(value, u.Quantity):
-            # This includes Distances as well
-            if z is not None or distmod is not None:
-                raise ValueError('`value` was given along with `z` or `distmod`'
-                                 ' in Quantity constructor.')
+        if z is not None:
+            if value is not None or distmod is not None:
+                raise ValueError('Should given only one of `value`, `z` '
+                                 'or `distmod` in Distance constructor.')
 
-            if unit is not None:
-                value = value.to(unit).value
-            else:
-                unit = value.unit
-                value = value.value
-        elif value is None:
-            if z is not None:
-                if distmod is not None:
-                    raise ValueError('both `z` and `distmod` given in Distance '
-                                     'constructor')
+            if cosmology is None:
+                from ..cosmology import default_cosmology
+                cosmology = default_cosmology.get()
 
-                if cosmology is None:
-                    cosmology = get_current()
+            value = cosmology.luminosity_distance(z)
+            # Continue on to take account of unit and other arguments
+            # but a copy is already made, so no longer necessary
+            copy = False
 
-                ld = cosmology.luminosity_distance(z)
+        else:
+            if cosmology is not None:
+                raise ValueError('A `cosmology` was given but `z` was not '
+                                 'provided in Distance constructor')
 
-                if unit is not None:
-                    ld = ld.to(unit)
-                value = ld.value
-                unit = ld.unit
+            if distmod is not None:
+                if value is not None:
+                    raise ValueError('Should given only one of `value`, `z` '
+                                     'or `distmod` in Distance constructor.')
 
-            elif distmod is not None:
                 value = cls._distmod_to_pc(distmod)
                 if unit is None:
-                    # choose unit based on most reasonable of Mpc, kpc, or pc
-                    if value > 1e6:
-                        value = value / 1e6
-                        unit = u.megaparsec
-                    elif value > 1e3:
-                        value = value / 1e3
-                        unit = u.kiloparsec
+                    # if the unit is not specified, guess based on the mean of
+                    # the log of the distance
+                    meanlogval = np.log10(value.value).mean()
+                    if meanlogval > 6:
+                        unit = u.Mpc
+                    elif meanlogval > 3:
+                        unit = u.kpc
+                    elif meanlogval < -3: #~200 AU
+                        unit = u.AU
                     else:
-                        unit = u.parsec
-                else:
-                    value = u.Quantity(value, u.parsec).to(unit).value
-            else:
-                raise ValueError('none of `value`, `z`, or `distmod` were given'
-                                 ' to Distance constructor')
+                        unit = u.pc
 
-                value = ld.value
-                unit = ld.unit
-        elif z is not None:  # and value is not None based on above
-            raise ValueError('Both `z` and a `value` were provided in Distance '
-                             'constructor')
-        elif cosmology is not None:
-            raise ValueError('A `cosmology` was given but `z` was not provided '
-                             'in Distance constructor')
-        elif unit is None:
-            raise u.UnitsError('No unit was provided for Distance')
-        #"else" the baseline `value` + `unit` case
+                # Continue on to take account of unit and other arguments
+                # but a copy is already made, so no longer necessary
+                copy = False
 
-        unit = _convert_to_and_validate_length_unit(unit)
+            elif value is None:
+                raise ValueError('None of `value`, `z`, or `distmod` were '
+                                 'given to Distance constructor')
 
-        try:
-            value = np.asarray(value)
-        except ValueError as e:
-            raise TypeError(str(e))
+        # now we have arguments like for a Quantity, so let it do the work
+        distance = super(Distance, cls).__new__(
+            cls, value, unit, dtype=dtype, copy=copy, order=order,
+            subok=subok, ndmin=ndmin)
 
-        if value.dtype.kind not in 'iuf':
-            raise TypeError("Unsupported dtype '{0}'".format(value.dtype))
+        if not distance.unit.is_equivalent(u.m):
+            raise u.UnitsError('Unit "{0}" is not a length type'.format(unit))
 
-        return super(Distance, cls).__new__(cls, value, unit, dtype=dtype,
-                                            copy=copy)
+        if not allow_negative and np.any(distance.value < 0):
+            raise ValueError("Distance must be >= 0.  Use the argument "
+                             "'allow_negative=True' to allow negative values.")
 
-    def __quantity_view__(self, obj, unit):
-        try:
-            unit = _convert_to_and_validate_length_unit(unit)
-            return obj.view(self.__class__)
-        except u.UnitsError:
-            return super(Distance, self).__quantity_view__(obj, unit)
+        return distance
 
-    def __quantity_instance__(self, val, unit, **kwargs):
-        try:
-            unit = _convert_to_and_validate_length_unit(unit)
-            return self.__class__(val, unit=unit, **kwargs)
-        except u.UnitsError:
-            return super(Distance, self).__quantity_instance__(
-                val, unit, **kwargs)
+    def __quantity_subclass__(self, unit):
+        if unit.is_equivalent(u.m):
+            return Distance, True
+        else:
+            return super(Distance, self).__quantity_subclass__(unit)[0], False
 
     @property
     def z(self):
@@ -179,14 +176,14 @@ class Distance(u.Quantity):
 
         Parameters
         ----------
-        cosmology : `~astropy.cosmology.cosmology` or None
-            The cosmology to assume for this calculation, or None to use the
-            current cosmology.
+        cosmology : ``Cosmology`` or `None`
+            The cosmology to assume for this calculation, or `None` to use the
+            current cosmology (see `astropy.cosmology` for details).
 
         Returns
         -------
         z : float
-            The redshift of this distance given the provided `cosmology`.
+            The redshift of this distance given the provided ``cosmology``.
         """
         from ..cosmology import luminosity_distance
         from scipy import optimize
@@ -198,15 +195,17 @@ class Distance(u.Quantity):
 
     @property
     def distmod(self):
-        """  The distance modulus of this distance as a Quantity """
+        """The distance modulus as a `~astropy.units.Quantity`"""
         val = 5. * np.log10(self.to(u.pc).value) - 5.
         return u.Quantity(val, u.mag)
 
-    @staticmethod
-    def _distmod_to_pc(dm):
-        return 10 ** ((dm + 5) / 5.)
+    @classmethod
+    def _distmod_to_pc(cls, dm):
+        dm = u.Quantity(dm, u.mag)
+        return cls(10 ** ((dm.value + 5) / 5.), u.pc, copy=False)
 
 
+@deprecated('v0.4', alternative='astropy.coordinates.CartesianRepresentation')
 class CartesianPoints(u.Quantity):
     """
     A cartesian representation of a point in three-dimensional space.
@@ -220,22 +219,23 @@ class CartesianPoints(u.Quantity):
         The second cartesian coordinate.
     z : `~astropy.units.Quantity` or array-like, optional
         The third cartesian coordinate.
-    unit : `~astropy.units.UnitBase` object or None
-        The physical unit of the coordinate values. If `x`, `y`, or `z`
+    unit : `~astropy.units.UnitBase` object or `None`
+        The physical unit of the coordinate values. If ``x``, ``y``, or ``z``
         are quantities, they will be converted to this unit.
-    dtype : ~numpy.dtype, optional
+    dtype : `~numpy.dtype`, optional
         See `~astropy.units.Quantity`. Must be given as a keyword argument.
     copy : bool, optional
         See `~astropy.units.Quantity`. Must be given as a keyword argument.
 
     Raises
     ------
-    astropy.units.UnitsError
-        If the units on `x`, `y`, and `z` do not match or an invalid unit is given
+    UnitsError
+        If the units on ``x``, ``y``, and ``z`` do not match or an invalid
+        unit is given.
     ValueError
-        If `y` and `z` don't match `x`'s shape or `x` is not length-3
+        If ``y`` and ``z`` don't match ``x``'s shape or ``x`` is not length-3
     TypeError
-        If incompatible array types are passed into `x`, `y`, or `z`
+        If incompatible array types are passed into ``x``, ``y``, or ``z``
 
     """
 
@@ -246,7 +246,7 @@ class CartesianPoints(u.Quantity):
     def __new__(cls, x, y=None, z=None, unit=None, dtype=None, copy=True):
         if y is None and z is None:
             if len(x) != 3:
-                raise ValueError('input to CartesianPoints is not length 3')
+                raise ValueError('Input to CartesianPoints is not length 3')
 
             qarr = x
             if unit is None and hasattr(qarr, 'unit'):
@@ -258,7 +258,7 @@ class CartesianPoints(u.Quantity):
                     if hasattr(coo, 'unit'):
                         if unit is not None and coo.unit != unit:
                             raise u.UnitsError('Units for `x`, `y`, and `z` do '
-                                               'not match in CartesianPoints')
+                                               'not match in CartesianPoints   ')
                         unit = coo.unit
                 #if `unit`  is still None at this point, it means none were
                 #Quantties, which is fine, because it means the user wanted
@@ -274,11 +274,11 @@ class CartesianPoints(u.Quantity):
 
             qarr = [np.asarray(coo) for coo in (x, y, z)]
             if not (qarr[0].shape == qarr[1].shape == qarr[2].shape):
-                raise ValueError("shapes for x,y, and z don't match in "
+                raise ValueError("Shapes for `x`, `y`, and `z` don't match in "
                                  "CartesianPoints")
                 #let the unit be whatever it is
         else:
-            raise TypeError('Must give all of x,y, and z or just array in '
+            raise TypeError('Must give all of `x`, `y`, and `z` or just array in '
                             'CartesianPoints')
         try:
             unit = _convert_to_and_validate_length_unit(unit, True)
@@ -293,47 +293,45 @@ class CartesianPoints(u.Quantity):
         if qarr.dtype.kind not in 'iuf':
             raise TypeError("Unsupported dtype '{0}'".format(qarr.dtype))
 
-        return super(CartesianPoints, cls).__new__(cls, qarr, unit, dtype=dtype,
-                                            copy=copy)
+        return u.Quantity.__new__(cls, qarr, unit, dtype=dtype, copy=copy)
 
-    def __quantity_view__(self, obj, unit):
-        unit = _convert_to_and_validate_length_unit(unit, True)
-        return super(CartesianPoints, self).__quantity_view__(obj, unit)
-
-    def __quantity_instance__(self, val, unit, **kwargs):
-        unit = _convert_to_and_validate_length_unit(unit, True)
-        return super(CartesianPoints, self).__quantity_instance__(val, unit, **kwargs)
+    def __quantity_subclass__(self, unit):
+        if unit.is_equivalent(u.m):
+            return CartesianPoints, True
+        else:
+            return u.Quantity.__quantity_subclass__(unit)[0], False
 
     def __array_wrap__(self, obj, context=None):
         #always convert to CartesianPoints because all operations that would
         #screw up the units are killed by _convert_to_and_validate_length_unit
-        obj = super(CartesianPoints, self).__array_wrap__(obj, context=context)
+        obj = u.Quantity.__array_wrap__(obj, context=context)
 
-        #always prefer self's unit
-        obj = obj.to(self.unit)
-
-        return CartesianPoints(obj.value, unit=obj.unit, copy=False)
+        #always prefer self's unit, if possible
+        if obj.unit.is_equivalent(self.unit):
+            return obj.to(self.unit)
+        else:
+            return obj
 
     @property
     def x(self):
         """
         The second cartesian coordinate as a `~astropy.units.Quantity`.
         """
-        return self[0]
+        return self.view(u.Quantity)[0]
 
     @property
     def y(self):
         """
         The second cartesian coordinate as a `~astropy.units.Quantity`.
         """
-        return self[1]
+        return self.view(u.Quantity)[1]
 
     @property
     def z(self):
         """
         The third cartesian coordinate as a `~astropy.units.Quantity`.
         """
-        return self[2]
+        return self.view(u.Quantity)[2]
 
     def to_spherical(self):
         """
@@ -341,11 +339,11 @@ class CartesianPoints(u.Quantity):
 
         Returns
         -------
-        r : astropy.units.Quantity
-            The radial coordinate (in the same units as this `CartesianPoint`).
-        lat : astropy.units.Quantity
+        r : `~astropy.units.Quantity`
+            The radial coordinate (in the same units as this `CartesianPoints`).
+        lat : `~astropy.units.Quantity`
             The spherical coordinates latitude.
-        lon : astropy.units.Quantity
+        lon : `~astropy.units.Quantity`
             The spherical coordinates longitude.
 
         """
@@ -362,7 +360,7 @@ class CartesianPoints(u.Quantity):
 
 def _convert_to_and_validate_length_unit(unit, allow_dimensionless=False):
     """
-    raises `astropy.units.UnitsError` if not a length unit
+    raises UnitsError if not a length unit
     """
     try:
         unit = u.Unit(unit)
