@@ -13,6 +13,7 @@ def get_package_data():
                                    't/bad.txt',
                                    't/bars_at_ends.txt',
                                    't/cds.dat',
+                                   't/cds_malformed.dat',
                                    't/cds/glob/ReadMe',
                                    't/cds/glob/lmxbrefs.dat',
                                    't/cds/multi/ReadMe',
@@ -24,9 +25,13 @@ def get_package_data():
                                    't/continuation.dat',
                                    't/daophot.dat',
                                    't/daophot2.dat',
+                                   't/daophot3.dat',
                                    't/sextractor.dat',
+                                   't/sextractor2.dat',
                                    't/daophot.dat.gz',
                                    't/fill_values.txt',
+                                   't/html.html',
+                                   't/html2.html',
                                    't/ipac.dat',
                                    't/ipac.dat.bz2',
                                    't/latex1.tex',
@@ -55,5 +60,11 @@ def get_package_data():
                                    't/test5.dat',
                                    't/vots_spec.dat',
                                    't/whitespace.dat',
+                                   't/simple_csv.csv',
+                                   't/simple_csv_missing.csv',
                                    ]
     }
+
+
+def requires_2to3():
+    return False
