@@ -1,18 +1,21 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+
+# TEST_UNICODE_LITERALS
+
 import re
 
 import numpy as np
 
 from ....utils import OrderedDict
 from ....tests.helper import pytest
-from ... import ascii as asciitable  # TODO: delete this line, use ascii.*
 from ... import ascii
 from ....table import Table
+from ....units import Unit
 from distutils import version
 
 from .common import (raises, assert_equal, assert_almost_equal,
                      assert_true, setup_function, teardown_function)
-
+from ....tests.helper import pytest
 
 _NUMPY_VERSION = version.LooseVersion(np.__version__)
 
@@ -104,7 +107,7 @@ def test_read_all_files():
             test_opts = testfile['opts'].copy()
             if 'guess' not in test_opts:
                 test_opts['guess'] = guess
-            table = asciitable.read(testfile['name'], **test_opts)
+            table = ascii.read(testfile['name'], **test_opts)
             assert_equal(table.dtype.names, testfile['cols'])
             for colname in table.dtype.names:
                 assert_equal(len(table[colname]), testfile['nrows'])
@@ -143,7 +146,7 @@ def test_guess_all_files():
             # Copy read options except for those in filter_read_opts
             guess_opts = dict((k, v) for k, v in testfile['opts'].items()
                               if k not in filter_read_opts)
-            table = asciitable.read(testfile['name'], guess=True, **guess_opts)
+            table = ascii.read(testfile['name'], guess=True, **guess_opts)
             assert_equal(table.dtype.names, testfile['cols'])
             for colname in table.dtype.names:
                 assert_equal(len(table[colname]), testfile['nrows'])
@@ -151,7 +154,7 @@ def test_guess_all_files():
 
 def test_daophot_indef():
     """Test that INDEF is correctly interpreted as a missing value"""
-    table = asciitable.read('t/daophot2.dat', Reader=asciitable.Daophot)
+    table = ascii.read('t/daophot2.dat', Reader=ascii.Daophot)
     for colname in table.colnames:
         # Three columns have all INDEF values and are masked
         mask_value = colname in ('OTIME', 'MAG', 'MERR', 'XAIRMASS')
@@ -164,7 +167,7 @@ def test_daophot_types():
     inferred automatically based only data values.  DAOphot reader uses
     the header information to assign types.
     """
-    table = asciitable.read('t/daophot2.dat', Reader=asciitable.Daophot)
+    table = ascii.read('t/daophot2.dat', Reader=ascii.Daophot)
     assert table['LID'].dtype.char in 'fd'  # float or double
     assert table['MAG'].dtype.char in 'fd'  # even without any data values
     assert table['PIER'].dtype.char in 'US'  # string (data values are consistent with int)
@@ -172,7 +175,7 @@ def test_daophot_types():
 
 
 def test_daophot_header_keywords():
-    table = asciitable.read('t/daophot.dat', Reader=asciitable.Daophot)
+    table = ascii.read('t/daophot.dat', Reader=ascii.Daophot)
     expected_keywords = (('NSTARFILE', 'test.nst.1', 'filename', '%-23s'),
                          ('REJFILE', '"hello world"', 'filename', '%-23s'),
                          ('SCALE', '1.',  'units/pix', '%-23.7g'),)
@@ -185,61 +188,69 @@ def test_daophot_header_keywords():
         assert_equal(keyword['format'], format_)
 
 
-@raises(asciitable.InconsistentTableError)
+def test_daophot_multiple_aperture():
+    table = ascii.read('t/daophot3.dat', Reader=ascii.Daophot)
+    assert 'MAG5' in table.colnames  # MAG5 is one of the newly created column name
+    assert table['MAG5'][4] == 22.13  # A sample entry in daophot3.dat file
+    assert table['MERR2'][0] == 1.171
+    assert np.all(table['RAPERT5'] == 23.3)  # assert all the 5th apertures are same 23.3
+
+
+@raises(ascii.InconsistentTableError)
 def test_empty_table_no_header():
-    table = asciitable.read('t/no_data_without_header.dat', Reader=asciitable.NoHeader,
+    table = ascii.read('t/no_data_without_header.dat', Reader=ascii.NoHeader,
                             guess=False)
 
 
-@raises(asciitable.InconsistentTableError)
+@raises(ascii.InconsistentTableError)
 def test_wrong_quote():
-    table = asciitable.read('t/simple.txt', guess=False)
+    table = ascii.read('t/simple.txt', guess=False)
 
 
-@raises(asciitable.InconsistentTableError)
+@raises(ascii.InconsistentTableError)
 def test_extra_data_col():
-    table = asciitable.read('t/bad.txt')
+    table = ascii.read('t/bad.txt')
 
 
-@raises(asciitable.InconsistentTableError)
+@raises(ascii.InconsistentTableError)
 def test_extra_data_col2():
-    table = asciitable.read('t/simple5.txt', delimiter='|')
+    table = ascii.read('t/simple5.txt', delimiter='|')
 
 
 @raises(IOError)
 def test_missing_file():
-    table = asciitable.read('does_not_exist')
+    table = ascii.read('does_not_exist')
 
 
 def test_set_names():
     names = ('c1', 'c2', 'c3', 'c4', 'c5', 'c6')
-    data = asciitable.read('t/simple3.txt', names=names, delimiter='|')
+    data = ascii.read('t/simple3.txt', names=names, delimiter='|')
     assert_equal(data.dtype.names, names)
 
 
 def test_set_include_names():
     names = ('c1', 'c2', 'c3', 'c4', 'c5', 'c6')
     include_names = ('c1', 'c3')
-    data = asciitable.read('t/simple3.txt', names=names, include_names=include_names,
+    data = ascii.read('t/simple3.txt', names=names, include_names=include_names,
                            delimiter='|')
     assert_equal(data.dtype.names, include_names)
 
 
 def test_set_exclude_names():
     exclude_names = ('Y', 'object')
-    data = asciitable.read('t/simple3.txt', exclude_names=exclude_names, delimiter='|')
+    data = ascii.read('t/simple3.txt', exclude_names=exclude_names, delimiter='|')
     assert_equal(data.dtype.names, ('obsid', 'redshift', 'X', 'rad'))
 
 
 def test_include_names_daophot():
     include_names = ('ID', 'MAG', 'PIER')
-    data = asciitable.read('t/daophot.dat', include_names=include_names)
+    data = ascii.read('t/daophot.dat', include_names=include_names)
     assert_equal(data.dtype.names, include_names)
 
 
 def test_exclude_names_daophot():
     exclude_names = ('ID', 'YCENTER', 'MERR', 'NITER', 'CHI', 'PERROR')
-    data = asciitable.read('t/daophot.dat', exclude_names=exclude_names)
+    data = ascii.read('t/daophot.dat', exclude_names=exclude_names)
     assert_equal(data.dtype.names, ('XCENTER', 'MAG', 'MSKY', 'SHARPNESS', 'PIER'))
 
 
@@ -248,7 +259,7 @@ def test_custom_process_lines():
         bars_at_ends = re.compile(r'^\| | \|$', re.VERBOSE)
         striplines = (x.strip() for x in lines)
         return [bars_at_ends.sub('', x) for x in striplines if len(x) > 0]
-    reader = asciitable.get_reader(delimiter='|')
+    reader = ascii.get_reader(delimiter='|')
     reader.inputter.process_lines = process_lines
     data = reader.read('t/bars_at_ends.txt')
     assert_equal(data.dtype.names, ('obsid', 'redshift', 'X', 'Y', 'object', 'rad'))
@@ -259,7 +270,7 @@ def test_custom_process_line():
     def process_line(line):
         line_out = re.sub(r'^\|\s*', '', line.strip())
         return line_out
-    reader = asciitable.get_reader(data_start=2, delimiter='|')
+    reader = ascii.get_reader(data_start=2, delimiter='|')
     reader.header.splitter.process_line = process_line
     reader.data.splitter.process_line = process_line
     data = reader.read('t/nls1_stackinfo.dbout')
@@ -268,9 +279,9 @@ def test_custom_process_line():
 
 
 def test_custom_splitters():
-    reader = asciitable.get_reader()
-    reader.header.splitter = asciitable.BaseSplitter()
-    reader.data.splitter = asciitable.BaseSplitter()
+    reader = ascii.get_reader()
+    reader.header.splitter = ascii.BaseSplitter()
+    reader.data.splitter = ascii.BaseSplitter()
     f = 't/test4.dat'
     data = reader.read(f)
     testfile = get_testfiles(f)
@@ -284,18 +295,18 @@ def test_custom_splitters():
 
 
 def test_start_end():
-    data = asciitable.read('t/test5.dat', header_start=1, data_start=3, data_end=-5)
+    data = ascii.read('t/test5.dat', header_start=1, data_start=3, data_end=-5)
     assert_equal(len(data), 13)
     assert_equal(data.field('statname')[0], 'chi2xspecvar')
     assert_equal(data.field('statname')[-1], 'chi2gehrels')
 
 
 def test_set_converters():
-    converters = {'zabs1.nh': [asciitable.convert_numpy('int32'),
-                               asciitable.convert_numpy('float32')],
-                  'p1.gamma': [asciitable.convert_numpy('str')]
+    converters = {'zabs1.nh': [ascii.convert_numpy('int32'),
+                               ascii.convert_numpy('float32')],
+                  'p1.gamma': [ascii.convert_numpy('str')]
                   }
-    data = asciitable.read('t/test4.dat', converters=converters)
+    data = ascii.read('t/test4.dat', converters=converters)
     assert_equal(str(data['zabs1.nh'].dtype), 'float32')
     assert_equal(data['p1.gamma'][0], '1.26764544642')
 
@@ -305,7 +316,7 @@ def test_from_string():
     with open(f) as fd:
         table = fd.read()
     testfile = get_testfiles(f)
-    data = asciitable.read(table, **testfile['opts'])
+    data = ascii.read(table, **testfile['opts'])
     assert_equal(data.dtype.names, testfile['cols'])
     assert_equal(len(data), testfile['nrows'])
 
@@ -314,7 +325,7 @@ def test_from_filelike():
     f = 't/simple.txt'
     testfile = get_testfiles(f)
     with open(f, 'rb') as fd:
-        data = asciitable.read(fd, **testfile['opts'])
+        data = ascii.read(fd, **testfile['opts'])
     assert_equal(data.dtype.names, testfile['cols'])
     assert_equal(len(data), testfile['nrows'])
 
@@ -324,13 +335,13 @@ def test_from_lines():
     with open(f) as fd:
         table = fd.readlines()
     testfile = get_testfiles(f)
-    data = asciitable.read(table, **testfile['opts'])
+    data = ascii.read(table, **testfile['opts'])
     assert_equal(data.dtype.names, testfile['cols'])
     assert_equal(len(data), testfile['nrows'])
 
 
 def test_comment_lines():
-    table = asciitable.get_reader(Reader=asciitable.Rdb)
+    table = ascii.get_reader(Reader=ascii.Rdb)
     data = table.read('t/apostrophe.rdb')
     assert_equal(table.comment_lines, ['# first comment', '  # second comment'])
 
@@ -338,7 +349,7 @@ def test_comment_lines():
 def test_fill_values():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, fill_values=('a', '1'), **testfile['opts'])
+    data = ascii.read(f, fill_values=('a', '1'), **testfile['opts'])
     assert_true((data['a'].mask == [False, True]).all())
     assert_true((data['a'] == [1, 1]).all())
     assert_true((data['b'].mask == [False, True]).all())
@@ -348,14 +359,14 @@ def test_fill_values():
 def test_fill_values_col():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, fill_values=('a', '1', 'b'), **testfile['opts'])
+    data = ascii.read(f, fill_values=('a', '1', 'b'), **testfile['opts'])
     check_fill_values(data)
 
 
 def test_fill_values_include_names():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, fill_values=('a', '1'),
+    data = ascii.read(f, fill_values=('a', '1'),
                            fill_include_names = ['b'], **testfile['opts'])
     check_fill_values(data)
 
@@ -363,7 +374,7 @@ def test_fill_values_include_names():
 def test_fill_values_exclude_names():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, fill_values=('a', '1'),
+    data = ascii.read(f, fill_values=('a', '1'),
                            fill_exclude_names = ['a'], **testfile['opts'])
     check_fill_values(data)
 
@@ -382,7 +393,7 @@ def check_fill_values(data):
 def test_fill_values_list():
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, fill_values=[('a', '42'), ('1', '42', 'a')],
+    data = ascii.read(f, fill_values=[('a', '42'), ('1', '42', 'a')],
                            **testfile['opts'])
     data['a'].mask = False  # explicitly unmask for comparison
     assert_true((data['a'] == [42, 42]).all())
@@ -391,7 +402,7 @@ def test_fill_values_list():
 def test_masking_Cds():
     f = 't/cds.dat'
     testfile = get_testfiles(f)
-    data = asciitable.read(f,
+    data = ascii.read(f,
                            **testfile['opts'])
     assert_true(data['AK'].mask[0])
     assert_true(not data['Fit'].mask[0])
@@ -400,11 +411,14 @@ def test_masking_Cds():
 def test_null_Ipac():
     f = 't/ipac.dat'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, **testfile['opts'])
+    data = ascii.read(f, **testfile['opts'])
     mask = np.array([(True, False, True, False, True),
                      (False, False, False, False, False)],
-                    dtype=[('ra', '|b1'), ('dec', '|b1'), ('sai', '|b1'),
-                           ('v2', '|b1'), ('sptype', '|b1')])
+                    dtype=[(str('ra'), '|b1'),
+                           (str('dec'), '|b1'),
+                           (str('sai'), '|b1'),
+                           (str('v2'), '|b1'),
+                           (str('sptype'), '|b1')])
     assert np.all(data.mask == mask)
 
 
@@ -416,7 +430,7 @@ def test_Ipac_meta():
     comments = ['This is an example of a valid comment']
     f = 't/ipac.dat'
     testfile = get_testfiles(f)
-    data = asciitable.read(f, **testfile['opts'])
+    data = ascii.read(f, **testfile['opts'])
     assert data.meta['keywords'].keys() == keywords.keys()
     for data_kv, kv in zip(data.meta['keywords'].values(), keywords.values()):
         assert data_kv['value'] == kv
@@ -425,19 +439,19 @@ def test_Ipac_meta():
 
 def test_set_guess_kwarg():
     """Read a file using guess with one of the typical guess_kwargs explicitly set."""
-    data = asciitable.read('t/space_delim_no_header.dat',
+    data = ascii.read('t/space_delim_no_header.dat',
                            delimiter=',', guess=True)
     assert(data.dtype.names == ('1 3.4 hello',))
     assert(len(data) == 1)
 
 
-@raises(asciitable.InconsistentTableError)
+@raises(ascii.InconsistentTableError)
 def test_read_rdb_wrong_type():
     """Read RDB data with inconstent data type (except failure)"""
     table = """col1\tcol2
 N\tN
 1\tHello"""
-    asciitable.read(table, Reader=asciitable.Rdb)
+    ascii.read(table, Reader=ascii.Rdb)
 
 
 def test_default_missing():
@@ -445,7 +459,7 @@ def test_default_missing():
     table = '\n'.join(['a,b,c,d',
                        '1,3,,',
                        '2, , 4.0 , ss '])
-    dat = asciitable.read(table)
+    dat = ascii.read(table)
     assert dat.masked is True
     assert dat.pformat() == [' a   b   c   d ',
                              '--- --- --- ---',
@@ -454,7 +468,7 @@ def test_default_missing():
 
     # Single row table with a single missing element
     table = """ a \n "" """
-    dat = asciitable.read(table)
+    dat = ascii.read(table)
     assert dat.pformat() == [' a ',
                              '---',
                              ' --']
@@ -465,14 +479,21 @@ def test_default_missing():
                        '--- --- --- ---',
                        '  1   3        ',
                        '  2     4.0  ss'])
-    dat = asciitable.read(table, Reader=asciitable.FixedWidthTwoLine)
+    dat = ascii.read(table, Reader=ascii.FixedWidthTwoLine)
     assert dat.masked is True
     assert dat.pformat() == [' a   b   c   d ',
                              '--- --- --- ---',
                              '  1   3  --  --',
                              '  2  -- 4.0  ss']
 
-    dat = asciitable.read(table, Reader=asciitable.FixedWidthTwoLine, fill_values=None)
+    dat = ascii.read(table, Reader=ascii.FixedWidthTwoLine, fill_values=None)
+    assert dat.masked is False
+    assert dat.pformat() == [' a   b   c   d ',
+                             '--- --- --- ---',
+                             '  1   3        ',
+                             '  2     4.0  ss']
+
+    dat = ascii.read(table, Reader=ascii.FixedWidthTwoLine, fill_values=[])
     assert dat.masked is False
     assert dat.pformat() == [' a   b   c   d ',
                              '--- --- --- ---',
@@ -488,11 +509,11 @@ def get_testfiles(name=None):
         {'cols': ('agasc_id', 'n_noids', 'n_obs'),
          'name': 't/apostrophe.rdb',
          'nrows': 2,
-         'opts': {'Reader': asciitable.Rdb}},
+         'opts': {'Reader': ascii.Rdb}},
         {'cols': ('agasc_id', 'n_noids', 'n_obs'),
          'name': 't/apostrophe.tab',
          'nrows': 2,
-         'opts': {'Reader': asciitable.Tab}},
+         'opts': {'Reader': ascii.Tab}},
         {'cols': ('Index',
                   'RAh',
                   'RAm',
@@ -507,20 +528,36 @@ def get_testfiles(name=None):
                   'Fit'),
          'name': 't/cds.dat',
          'nrows': 1,
-         'opts': {'Reader': asciitable.Cds}},
+         'opts': {'Reader': ascii.Cds}},
+        # Test malformed CDS file (issues #2241 #467)
+        {'cols': ('Index',
+                  'RAh',
+                  'RAm',
+                  'RAs',
+                  'DE-',
+                  'DEd',
+                  'DEm',
+                  'DEs',
+                  'Match',
+                  'Class',
+                  'AK',
+                  'Fit'),
+         'name': 't/cds_malformed.dat',
+         'nrows': 1,
+         'opts': {'Reader': ascii.Cds, 'data_start': 'guess'}},
         {'cols': ('a', 'b', 'c'),
          'name': 't/commented_header.dat',
          'nrows': 2,
-         'opts': {'Reader': asciitable.CommentedHeader}},
+         'opts': {'Reader': ascii.CommentedHeader}},
         {'cols': ('a', 'b', 'c'),
          'name': 't/commented_header2.dat',
          'nrows': 2,
-         'opts': {'Reader': asciitable.CommentedHeader, 'header_start': -1}},
+         'opts': {'Reader': ascii.CommentedHeader, 'header_start': -1, 'data_start': 0}},
         {'cols': ('col1', 'col2', 'col3', 'col4', 'col5'),
          'name': 't/continuation.dat',
          'nrows': 2,
-         'opts': {'Inputter': asciitable.ContinuationLinesInputter,
-                  'Reader': asciitable.NoHeader}},
+         'opts': {'Inputter': ascii.ContinuationLinesInputter,
+                  'Reader': ascii.NoHeader}},
         {'cols': ('ID',
                   'XCENTER',
                   'YCENTER',
@@ -534,8 +571,7 @@ def get_testfiles(name=None):
                   'PERROR'),
          'name': 't/daophot.dat',
          'nrows': 2,
-         'requires_numpy': True,
-         'opts': {'Reader': asciitable.Daophot}},
+         'opts': {'Reader': ascii.Daophot}},
         {'cols': ('NUMBER',
                   'FLUX_ISO',
                   'FLUXERR_ISO',
@@ -544,12 +580,11 @@ def get_testfiles(name=None):
                   'FLAG'),
          'name': 't/sextractor.dat',
          'nrows': 3,
-         'requires_numpy': True,
-         'opts': {'Reader': asciitable.SExtractor}},
+         'opts': {'Reader': ascii.SExtractor}},
         {'cols': ('ra', 'dec', 'sai', 'v2', 'sptype'),
          'name': 't/ipac.dat',
          'nrows': 2,
-         'opts': {'Reader': asciitable.Ipac}},
+         'opts': {'Reader': ascii.Ipac}},
         {'cols': ('col0',
                   'objID',
                   'osrcid',
@@ -586,7 +621,7 @@ def get_testfiles(name=None):
                   'Fit'),
          'name': 't/no_data_cds.dat',
          'nrows': 0,
-         'opts': {'Reader': asciitable.Cds}},
+         'opts': {'Reader': ascii.Cds}},
         {'cols': ('ID',
                   'XCENTER',
                   'YCENTER',
@@ -600,8 +635,7 @@ def get_testfiles(name=None):
                   'PERROR'),
          'name': 't/no_data_daophot.dat',
          'nrows': 0,
-         'requires_numpy': True,
-         'opts': {'Reader': asciitable.Daophot}},
+         'opts': {'Reader': ascii.Daophot}},
         {'cols': ('NUMBER',
                   'FLUX_ISO',
                   'FLUXERR_ISO',
@@ -610,16 +644,15 @@ def get_testfiles(name=None):
                   'FLAG'),
          'name': 't/no_data_sextractor.dat',
          'nrows': 0,
-         'requires_numpy': True,
-         'opts': {'Reader': asciitable.SExtractor}},
+         'opts': {'Reader': ascii.SExtractor}},
         {'cols': ('ra', 'dec', 'sai', 'v2', 'sptype'),
          'name': 't/no_data_ipac.dat',
          'nrows': 0,
-         'opts': {'Reader': asciitable.Ipac}},
+         'opts': {'Reader': ascii.Ipac}},
         {'cols': ('ra', 'v2'),
          'name': 't/ipac.dat',
          'nrows': 2,
-         'opts': {'Reader': asciitable.Ipac, 'include_names': ['ra', 'v2']}},
+         'opts': {'Reader': ascii.Ipac, 'include_names': ['ra', 'v2']}},
         {'cols': ('a', 'b', 'c'),
          'name': 't/no_data_with_header.dat',
          'nrows': 0,
@@ -627,15 +660,27 @@ def get_testfiles(name=None):
         {'cols': ('agasc_id', 'n_noids', 'n_obs'),
          'name': 't/short.rdb',
          'nrows': 7,
-         'opts': {'Reader': asciitable.Rdb}},
+         'opts': {'Reader': ascii.Rdb}},
         {'cols': ('agasc_id', 'n_noids', 'n_obs'),
          'name': 't/short.tab',
          'nrows': 7,
-         'opts': {'Reader': asciitable.Tab}},
+         'opts': {'Reader': ascii.Tab}},
         {'cols': ('test 1a', 'test2', 'test3', 'test4'),
          'name': 't/simple.txt',
          'nrows': 2,
          'opts': {'quotechar': "'"}},
+        {'cols': ('top1', 'top2', 'top3', 'top4'),
+         'name': 't/simple.txt',
+         'nrows': 1,
+         'opts': {'quotechar': "'", 'header_start': 1, 'data_start': 2}},
+        {'cols': ('top1', 'top2', 'top3', 'top4'),
+         'name': 't/simple.txt',
+         'nrows': 1,
+         'opts': {'quotechar': "'", 'header_start': 1}},
+        {'cols': ('top1', 'top2', 'top3', 'top4'),
+         'name': 't/simple.txt',
+         'nrows': 2,
+         'opts': {'quotechar': "'", 'header_start': 1, 'data_start': 1}},
         {'cols': ('obsid', 'redshift', 'X', 'Y', 'object', 'rad'),
          'name': 't/simple2.txt',
          'nrows': 3,
@@ -647,11 +692,15 @@ def get_testfiles(name=None):
         {'cols': ('col1', 'col2', 'col3', 'col4', 'col5', 'col6'),
          'name': 't/simple4.txt',
          'nrows': 3,
-         'opts': {'Reader': asciitable.NoHeader, 'delimiter': '|'}},
+         'opts': {'Reader': ascii.NoHeader, 'delimiter': '|'}},
         {'cols': ('col1', 'col2', 'col3'),
          'name': 't/space_delim_no_header.dat',
          'nrows': 2,
-         'opts': {'Reader': asciitable.NoHeader}},
+         'opts': {'Reader': ascii.NoHeader}},
+        {'cols': ('col1', 'col2', 'col3'),
+         'name': 't/space_delim_no_header.dat',
+         'nrows': 2,
+         'opts': {'Reader': ascii.NoHeader, 'header_start': None}},
         {'cols': ('obsid', 'offset', 'x', 'y', 'name', 'oaa'),
          'name': 't/space_delim_blank_lines.txt',
          'nrows': 3,
@@ -668,21 +717,62 @@ def get_testfiles(name=None):
          'cols': ('quoted colname with tab\tinside', 'col2', 'col3'),
          'nrows': 2,
          'opts': {'delimiter': '\s'}},
+        {'name': 't/simple_csv.csv',
+         'cols': ('a','b','c'),
+         'nrows': 2,
+         'opts': {'Reader': ascii.Csv}},
+        {'name': 't/simple_csv_missing.csv',
+         'cols': ('a','b','c'),
+         'nrows': 2,
+         'skip': True,
+         'opts': {'Reader': ascii.Csv}},
         {'cols': ('cola', 'colb', 'colc'),
          'name': 't/latex1.tex',
          'nrows': 2,
-         'opts': {'Reader': asciitable.Latex}},
+         'opts': {'Reader': ascii.Latex}},
         {'cols': ('Facility', 'Id', 'exposure', 'date'),
          'name': 't/latex2.tex',
          'nrows': 3,
-         'opts': {'Reader': asciitable.AASTex}},
+         'opts': {'Reader': ascii.AASTex}},
     ]
 
+    try:
+        import bs4
+        testfiles.append({'cols': ('Column 1', 'Column 2', 'Column 3'),
+                          'name': 't/html.html',
+                          'nrows': 3,
+                          'opts': {'Reader': ascii.HTML}})
+    except ImportError:
+        pass
+
     if name is not None:
         return [x for x in testfiles if x['name'] == name][0]
     else:
         return testfiles
 
+def test_header_start_exception():
+    '''Check certain Readers throw an exception if ``header_start`` is set
+
+    For certain Readers it does not make sense to set the ``header_start``, they
+    throw an exception if you try.
+    This was implemented in response to issue #885.
+    '''
+    for readerclass in [ascii.NoHeader, ascii.SExtractor, ascii.Ipac,
+                   ascii.BaseReader, ascii.FixedWidthNoHeader, ascii.Cds, ascii.Daophot]:
+        with pytest.raises(ValueError):
+            reader = ascii.core._get_reader(readerclass, header_start=5)
+
+
+def test_csv_table_read():
+    """
+    Check for a regression introduced by #1935.  Pseudo-CSV file with
+    commented header line.
+    """
+    lines = ['# a, b',
+             '1, 2',
+             '3, 4']
+    t = ascii.read(lines)
+    assert t.colnames == ['a', 'b']
 
 def test_overlapping_names():
     """
@@ -691,3 +781,35 @@ def test_overlapping_names():
     """
     t = ascii.read(['a b', '1 2'], names=['b', 'a'])
     assert t.colnames == ['b', 'a']
+
+def test_sextractor_units():
+    """
+    Make sure that the SExtractor reader correctly inputs descriptions and units.
+    """
+    table = ascii.read('t/sextractor2.dat', Reader=ascii.SExtractor, guess=False)
+    expected_units = [None, Unit('pix'), Unit('pix'), Unit('mag'),
+                Unit('mag'), None, Unit('pix**2'), Unit('m**(-6)'),
+                Unit('mag * arcsec**(-2)')]
+    expected_descrs = ['Running object number',
+                       'Windowed position estimate along x',
+                       'Windowed position estimate along y',
+                       'Kron-like elliptical aperture magnitude',
+                       'RMS error for AUTO magnitude',
+                       'Extraction flags',
+                       None,
+                       'Barycenter position along MAMA x axis',
+                       'Peak surface brightness above background']
+    for i, colname in enumerate(table.colnames):
+        assert table[colname].unit == expected_units[i]
+        assert table[colname].description == expected_descrs[i]
+
+def test_list_with_newlines():
+    """
+    Check that lists of strings where some strings consist of just a newline
+    ("\n") are parsed correctly.
+    """
+    t = ascii.read(["abc", "123\n", "456\n", "\n", "\n"])
+    assert t.colnames == ['abc']
+    assert len(t) == 2
+    assert t[0][0] == 123
+    assert t[1][0] == 456
