@@ -6,17 +6,18 @@ import gzip
 import mmap
 import os
 import tempfile
-import urllib
 import warnings
 import zipfile
 
+from functools import reduce
+
 import numpy as np
 from numpy import memmap as Memmap
 
 from .util import (isreadable, iswritable, isfile, fileobj_open, fileobj_name,
                    fileobj_closed, fileobj_mode, _array_from_file,
                    _array_to_file, _write_string, encode_ascii)
-from ...extern.six import PY3, b, string_types
+from ...extern.six import b, string_types
 from ...extern.six.moves import urllib
 from ...utils.exceptions import AstropyUserWarning
 
@@ -105,7 +106,7 @@ class _File(object):
         if mode not in PYFITS_MODES:
             raise ValueError("Mode '%s' not recognized" % mode)
 
-        if (isinstance(fileobj, basestring) and
+        if (isinstance(fileobj, string_types) and
                 mode not in ('ostream', 'append') and
                 not os.path.exists(fileobj)):
 
@@ -124,7 +125,7 @@ class _File(object):
             except (TypeError, ValueError, IOError):
                 # A couple different exceptions can occur here when passing a
                 # filename into urlretrieve in Python 3
-                raise IOError('File does not exist: %r' % fileobj)
+                raise IOError('File does not exist: {0!r}'.format(fileobj))
         else:
             self.name = fileobj_name(fileobj)
 
@@ -144,11 +145,13 @@ class _File(object):
         # Initialize the internal self.__file object
         if _is_random_access_file_backed(fileobj):
             self._open_fileobj(fileobj, mode, clobber)
-        elif isinstance(fileobj, basestring):
+        elif isinstance(fileobj, string_types):
             self._open_filename(fileobj, mode, clobber)
         else:
             self._open_filelike(fileobj, mode, clobber)
 
+        self.fileobj_mode = fileobj_mode(self.__file)
+
         if isinstance(fileobj, gzip.GzipFile):
             self.compression = 'gzip'
         elif isinstance(fileobj, zipfile.ZipFile):
@@ -477,6 +480,7 @@ class _File(object):
                 warnings.warn('Failed to create mmap: %s; mmap use will be '
                               'disabled' % str(e), AstropyUserWarning)
                 _File._mmap_available = False
+                del exc
                 return False
             try:
                 mm.flush()
