@@ -8,6 +8,8 @@ from ..header import Header
 from ..util import (_is_pseudo_unsigned, _unsigned_zero, _is_int,
                     _normalize_slice)
 
+from ....extern.six import string_types
+from ....extern.six.moves import xrange
 from ....utils import lazyproperty
 
 
@@ -42,7 +44,7 @@ class _ImageBaseHDU(_ValidHDU):
     }
 
     def __init__(self, data=None, header=None, do_not_scale_image_data=False,
-                 uint=False, scale_back=False, **kwargs):
+                 uint=False, scale_back=False, ignore_blank=False, **kwargs):
 
         from .groups import GroupsHDU
 
@@ -114,15 +116,16 @@ class _ImageBaseHDU(_ValidHDU):
         # Save off other important values from the header needed to interpret
         # the image data
         self._axes = [self._header.get('NAXIS' + str(axis + 1), 0)
-                      for axis in xrange(self._header.get('NAXIS', 0))]
+                      for axis in range(self._header.get('NAXIS', 0))]
         self._bitpix = self._header.get('BITPIX', 8)
         self._gcount = self._header.get('GCOUNT', 1)
         self._pcount = self._header.get('PCOUNT', 0)
-        self._blank = self._header.get('BLANK')
+        self._blank = None if ignore_blank else self._header.get('BLANK')
 
         self._orig_bitpix = self._bitpix
         self._orig_bzero = self._bzero
         self._orig_bscale = self._bscale
+        self._orig_blank = self._header.get('BLANK')
 
         # Set the name attribute if it was provided (if this is an ImageHDU
         # this will result in setting the EXTNAME keyword of the header as
@@ -658,7 +661,7 @@ class _ImageBaseHDU(_ValidHDU):
             else:
                 byteswapped = False
 
-            cs = self._compute_checksum(np.fromstring(d, dtype='ubyte'),
+            cs = self._compute_checksum(d.flatten().view(np.uint8),
                                         blocking=blocking)
 
             # If the data was byteswapped in this method then return it to
@@ -808,6 +811,7 @@ class PrimaryHDU(_ImageBaseHDU):
     _default_name = 'PRIMARY'
 
     def __init__(self, data=None, header=None, do_not_scale_image_data=False,
+                 ignore_blank=False,
                  uint=False, scale_back=None):
         """
         Construct a primary HDU.
@@ -825,11 +829,16 @@ class PrimaryHDU(_ImageBaseHDU):
             If `True`, image data is not scaled using BSCALE/BZERO values
             when read.
 
+        ignore_blank : bool, optional
+            If `True`, the BLANK header keyword will be ignored if present.
+            Otherwise, pixels equal to this value will be replaced with
+            NaNs.
+
         uint : bool, optional
             Interpret signed integer data where ``BZERO`` is the
             central value and ``BSCALE == 1`` as unsigned integer
-            data.  For example, `int16` data with ``BZERO = 32768``
-            and ``BSCALE = 1`` would be treated as `uint16` data.
+            data.  For example, ``int16`` data with ``BZERO = 32768``
+            and ``BSCALE = 1`` would be treated as ``uint16`` data.
 
         scale_back : bool, optional
             If `True`, when saving changes to a file that contained scaled
@@ -842,6 +851,7 @@ class PrimaryHDU(_ImageBaseHDU):
         super(PrimaryHDU, self).__init__(
             data=data, header=header,
             do_not_scale_image_data=do_not_scale_image_data, uint=uint,
+            ignore_blank=ignore_blank,
             scale_back=scale_back)
 
         # insert the keywords EXTEND
@@ -901,7 +911,7 @@ class ImageHDU(_ImageBaseHDU, ExtensionHDU):
             The data in the HDU.
 
         header : Header instance
-            The header to be used (as a template).  If `header` is
+            The header to be used (as a template).  If ``header`` is
             `None`, a minimal header will be provided.
 
         name : str, optional
@@ -915,8 +925,8 @@ class ImageHDU(_ImageBaseHDU, ExtensionHDU):
         uint : bool, optional
             Interpret signed integer data where ``BZERO`` is the
             central value and ``BSCALE == 1`` as unsigned integer
-            data.  For example, `int16` data with ``BZERO = 32768``
-            and ``BSCALE = 1`` would be treated as `uint16` data.
+            data.  For example, ``int16`` data with ``BZERO = 32768``
+            and ``BSCALE = 1`` would be treated as ``uint16`` data.
 
         scale_back : bool, optional
             If `True`, when saving changes to a file that contained scaled
@@ -938,7 +948,7 @@ class ImageHDU(_ImageBaseHDU, ExtensionHDU):
     def match_header(cls, header):
         card = header.cards[0]
         xtension = card.value
-        if isinstance(xtension, basestring):
+        if isinstance(xtension, string_types):
             xtension = xtension.rstrip()
         return card.keyword == 'XTENSION' and xtension == cls._extension
 
