@@ -1,18 +1,22 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see PYFITS.rst
 
-import itertools
-import sys
+
+from __future__ import division, with_statement
+
 import warnings
 
 from io import StringIO, BytesIO
 
 import numpy as np
 
+from ....extern import six
+from ....extern.six import u, iterkeys, itervalues, iteritems
+from ....extern.six.moves import zip
 from ....io import fits
 from ....io.fits.verify import VerifyWarning
-from ....extern.six import u
 from ....tests.helper import pytest, catch_warnings
+from ....utils.exceptions import AstropyDeprecationWarning
 
 from . import FitsTestCase
 from .util import ignore_warnings
@@ -21,9 +25,6 @@ from ..header import _pad_length, BLOCK_SIZE
 from ..util import encode_ascii
 
 
-PY3 = sys.version_info[0] >= 3
-
-
 class TestOldApiHeaderFunctions(FitsTestCase):
     """
     Tests that specifically use attributes and methods from the old
@@ -32,11 +33,13 @@ class TestOldApiHeaderFunctions(FitsTestCase):
     This tests backward compatibility support for those interfaces.
     """
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_ascardimage_verifies_the_comment_string_to_be_ascii_text(self):
         # the ascardimage() verifies the comment string to be ASCII text
         c = fits.Card.fromstring('abc     = +  2.1   e + 12 / abcde\0')
         pytest.raises(Exception, c.ascardimage)
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_rename_key(self):
         """Test backwards compatibility support for Header.rename_key()"""
         header = fits.Header([('A', 'B', 'C'), ('D', 'E', 'F')])
@@ -47,6 +50,7 @@ class TestOldApiHeaderFunctions(FitsTestCase):
         assert header['B'] == 'B'
         assert header.comments['B'] == 'C'
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_add_commentary(self):
         header = fits.Header([('A', 'B', 'C'), ('HISTORY', 1),
                               ('HISTORY', 2), ('HISTORY', 3), ('', '', ''),
@@ -78,6 +82,7 @@ class TestOldApiHeaderFunctions(FitsTestCase):
         assert header.cards[1].value == 0
         assert header[''] == [0, 1, 2, 3, '', '', 4]
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_totxtfile(self):
         hdul = fits.open(self.data('test0.fits'))
         hdul[0].header.toTxtFile(self.temp('header.txt'))
@@ -112,6 +117,7 @@ class TestOldApiHeaderFunctions(FitsTestCase):
         assert len(hdul2) == 2
         assert 'MYKEY' in hdul2[1].header
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_update_comment(self):
         hdul = fits.open(self.data('arange.fits'))
         hdul[0].header.update('FOO', 'BAR', 'BAZ')
@@ -127,6 +133,7 @@ class TestOldApiHeaderFunctions(FitsTestCase):
         hdul = fits.open(self.temp('test.fits'))
         assert hdul[0].header.ascard['FOO'].comment == 'QUX'
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_long_commentary_card(self):
         # Another version of this test using new API methods is found in
         # TestHeaderFunctions
@@ -148,6 +155,7 @@ class TestOldApiHeaderFunctions(FitsTestCase):
         assert str(header.cards[1]) == 'HISTORY ' + longval[:72]
         assert str(header.cards[2]).rstrip() == 'HISTORY ' + longval[72:]
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_wildcard_slice(self):
         """Test selecting a subsection of a header via wildcard matching."""
 
@@ -160,9 +168,10 @@ class TestOldApiHeaderFunctions(FitsTestCase):
         assert cards[0].value == 0
         assert cards[1].value == 2
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_assign_boolean(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/123
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/123
 
         Tests assigning Python and Numpy boolean values to keyword values.
         """
@@ -193,8 +202,9 @@ class TestOldApiHeaderFunctions(FitsTestCase):
         assert h.ascard['FOO'].cardimage == fooimg
         assert h.ascard['BAR'].cardimage == barimg
 
+    @ignore_warnings(AstropyDeprecationWarning)
     def test_cardlist_list_methods(self):
-        """Regression test for https://trac.assembla.com/pyfits/ticket/190"""
+        """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/190"""
 
         header = fits.Header()
         header.update('A', 'B', 'C')
@@ -222,7 +232,11 @@ class TestHeaderFunctions(FitsTestCase):
         """Test Card constructor with default argument values."""
 
         c = fits.Card()
-        assert '' == c.key
+
+        with ignore_warnings():
+            assert '' == c.key
+
+        assert '' == c.keyword
 
     def test_string_value_card(self):
         """Test Card constructor with string value"""
@@ -238,7 +252,7 @@ class TestHeaderFunctions(FitsTestCase):
         c = fits.Card("abc", True)
         assert str(c) == _pad("ABC     =                    T")
 
-        c = fits.Card.fromstring('abc     = F')
+        c = fits.Card.fromstring('ABC     = F')
         assert c.value == False
 
     def test_long_integer_value_card(self):
@@ -311,10 +325,10 @@ class TestHeaderFunctions(FitsTestCase):
     def test_commentary_cards(self):
         # commentary cards
         val = "A commentary card's value has no quotes around it."
-        c = fits.Card("history", val)
+        c = fits.Card("HISTORY", val)
         assert str(c) == _pad('HISTORY ' + val)
         val = "A commentary card has no comment."
-        c = fits.Card("comment", val, "comment")
+        c = fits.Card("COMMENT", val, "comment")
         assert str(c) == _pad('COMMENT ' + val)
 
     def test_commentary_card_created_by_fromstring(self):
@@ -328,13 +342,13 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_commentary_card_will_not_parse_numerical_value(self):
         # commentary card will not parse the numerical value
-        c = fits.Card.fromstring("history  (1, 2)")
+        c = fits.Card.fromstring("HISTORY  (1, 2)")
         assert str(c) == _pad("HISTORY  (1, 2)")
 
     def test_equal_sign_after_column8(self):
         # equal sign after column 8 of a commentary card will be part ofthe
         # string value
-        c = fits.Card.fromstring("history =   (1, 2)")
+        c = fits.Card.fromstring("HISTORY =   (1, 2)")
         assert str(c) == _pad("HISTORY =   (1, 2)")
 
     def test_blank_keyword(self):
@@ -351,8 +365,8 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_complex_number_using_string_input(self):
         # complex number using string input
-        c = fits.Card.fromstring('abc     = (8, 9)')
-        assert str(c) == _pad("ABC     =               (8, 9)")
+        c = fits.Card.fromstring('ABC     = (8, 9)')
+        assert str(c) == _pad("ABC     = (8, 9)")
 
     def test_fixable_non_standard_fits_card(self, capsys):
         # fixable non-standard FITS card will keep the original format
@@ -372,12 +386,12 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_undefined_value_using_string_input(self):
         # undefined value using string input
-        c = fits.Card.fromstring('abc     =    ')
+        c = fits.Card.fromstring('ABC     =    ')
         assert str(c) == _pad("ABC     =")
 
     def test_mislocated_equal_sign(self, capsys):
         # test mislocated "=" sign
-        c = fits.Card.fromstring('xyz= 100')
+        c = fits.Card.fromstring('XYZ= 100')
         assert c.keyword == 'XYZ'
         assert c.value == 100
         assert str(c) == _pad("XYZ     =                  100")
@@ -389,29 +403,29 @@ class TestHeaderFunctions(FitsTestCase):
         # something like "HISTO   = '=   (1, 2)".  But since ticket #109 if the
         # format is completely wrong we don't make any assumptions and the card
         # should be left alone
-        c = fits.Card.fromstring("histo       =   (1, 2)")
-        assert str(c) == _pad("histo       =   (1, 2)")
+        c = fits.Card.fromstring("HISTO       =   (1, 2)")
+        assert str(c) == _pad("HISTO       =   (1, 2)")
 
         # Likewise this card should just be left in its original form and
         # we shouldn't guess how to parse it or rewrite it.
-        c = fits.Card.fromstring("   history          (1, 2)")
-        assert str(c) == _pad("   history          (1, 2)")
+        c = fits.Card.fromstring("   HISTORY          (1, 2)")
+        assert str(c) == _pad("   HISTORY          (1, 2)")
 
     def test_verify_invalid_equal_sign(self):
         # verification
-        c = fits.Card.fromstring('abc= a6')
+        c = fits.Card.fromstring('ABC= a6')
         with catch_warnings() as w:
             c.verify()
         err_text1 = ("Card 'ABC' is not FITS standard (equal sign not at "
                      "column 8)")
         err_text2 = ("Card 'ABC' is not FITS standard (invalid value "
-                     "string: a6")
+                     "string: 'a6'")
         assert len(w) == 4
         assert err_text1 in str(w[1].message)
         assert err_text2 in str(w[2].message)
 
     def test_fix_invalid_equal_sign(self):
-        c = fits.Card.fromstring('abc= a6')
+        c = fits.Card.fromstring('ABC= a6')
         with catch_warnings() as w:
             c.verify('fix')
         fix_text = "Fixed 'ABC' card to meet the FITS standard."
@@ -443,13 +457,13 @@ class TestHeaderFunctions(FitsTestCase):
 
         h2 = fits.Header()
         with catch_warnings() as w:
-            h2['TEST'] = u('abcdefg' * 30)
+            h2['TEST'] = u('abcdefg') * 30
             assert len(w) == 0
 
         assert str(h1) == str(h2)
 
     def test_long_string_repr(self):
-        """Regression test for https://trac.assembla.com/pyfits/ticket/193
+        """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/193
 
         Ensure that the __repr__() for cards represented with CONTINUE cards is
         split across multiple lines (broken at each *physical* card).
@@ -471,7 +485,7 @@ class TestHeaderFunctions(FitsTestCase):
              str(fits.Card('TEST3', 'Regular value', 'Regular comment'))])
 
     def test_blank_keyword_long_value(self):
-        """Regression test for https://trac.assembla.com/pyfits/ticket/194
+        """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/194
 
         Test that a blank keyword ('') can be assigned a too-long value that is
         continued across multiple cards with blank keywords, just like COMMENT
@@ -518,7 +532,7 @@ class TestHeaderFunctions(FitsTestCase):
             "CONTINUE  '&' / longcommentlongcommentlongcommentlongcommentlongcommentlongcomme"
             "CONTINUE  '&' / ntlongcommentlongcommentlongcommentlongcomment                  ")
 
-    def test_long_string_value_via_fromstring(self):
+    def test_long_string_value_via_fromstring(self, capsys):
         # long string value via fromstring() method
         c = fits.Card.fromstring(
             _pad("abc     = 'longstring''s testing  &  ' "
@@ -534,7 +548,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_continue_card_with_equals_in_value(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/117
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/117
         """
 
         c = fits.Card.fromstring(
@@ -647,13 +661,13 @@ class TestHeaderFunctions(FitsTestCase):
         assert 'abcdefghi' in header
         assert header['abcdefghi'] == 10
         # This used to be assert_false, but per ticket
-        # https://trac.assembla.com/pyfits/ticket/155 hierarch keywords
+        # https://aeon.stsci.edu/ssb/trac/pyfits/ticket/155 hierarch keywords
         # should be treated case-insensitively when performing lookups
         assert 'ABCDEFGHI' in header
 
     def test_hierarch_create_and_update(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/158
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/158
 
         Tests several additional use cases for working with HIERARCH cards.
         """
@@ -714,7 +728,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_short_hierarch_create_and_update(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/158
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/158
 
         Tests several additional use cases for working with HIERARCH cards,
         specifically where the keyword is fewer than 8 characters, but contains
@@ -888,6 +902,17 @@ class TestHeaderFunctions(FitsTestCase):
         assert newheader[0] == 0
         assert newheader[1] == 2
 
+    def test_wildcard_with_hyphen(self):
+        """
+        Regression test for issue where wildcards did not work on keywords
+        containing hyphens.
+        """
+
+        header = fits.Header([('DATE', 1), ('DATE-OBS', 2), ('DATE-FOO', 3)])
+        assert len(header['DATE*']) == 3
+        assert len(header['DATE?*']) == 2
+        assert len(header['DATE-*']) == 2
+
     def test_wildcard_slice_assignment(self):
         """Test assigning to a header slice selected via wildcard matching."""
 
@@ -967,21 +992,21 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_header_items(self):
         header = fits.Header([('A', 'B'), ('C', 'D')])
-        assert header.items() == list(header.iteritems())
+        assert list(header.items()) == list(iteritems(header))
 
     def test_header_iterkeys(self):
         header = fits.Header([('A', 'B'), ('C', 'D')])
-        for a, b in itertools.izip(header.iterkeys(), header):
+        for a, b in zip(iterkeys(header), header):
             assert a == b
 
     def test_header_itervalues(self):
         header = fits.Header([('A', 'B'), ('C', 'D')])
-        for a, b in itertools.izip(header.itervalues(), ['B', 'D']):
+        for a, b in zip(itervalues(header), ['B', 'D']):
             assert a == b
 
     def test_header_keys(self):
         hdul = fits.open(self.data('arange.fits'))
-        assert (hdul[0].header.keys() ==
+        assert (list(hdul[0].header) ==
                 ['SIMPLE', 'BITPIX', 'NAXIS', 'NAXIS1', 'NAXIS2', 'NAXIS3',
                  'EXTEND'])
 
@@ -992,17 +1017,17 @@ class TestHeaderFunctions(FitsTestCase):
         last = header.pop()
         assert last == 'H'
         assert len(header) == 3
-        assert header.keys() == ['A', 'C', 'E']
+        assert list(header) == ['A', 'C', 'E']
 
         mid = header.pop(1)
         assert mid == 'D'
         assert len(header) == 2
-        assert header.keys() == ['A', 'E']
+        assert list(header) == ['A', 'E']
 
         first = header.pop(0)
         assert first == 'B'
         assert len(header) == 1
-        assert header.keys() == ['E']
+        assert list(header) == ['E']
 
         pytest.raises(IndexError, header.pop, 42)
 
@@ -1014,17 +1039,17 @@ class TestHeaderFunctions(FitsTestCase):
         last = header.pop('G')
         assert last == 'H'
         assert len(header) == 3
-        assert header.keys() == ['A', 'C', 'E']
+        assert list(header) == ['A', 'C', 'E']
 
         mid = header.pop('C')
         assert mid == 'D'
         assert len(header) == 2
-        assert header.keys() == ['A', 'E']
+        assert list(header) == ['A', 'E']
 
         first = header.pop('A')
         assert first == 'B'
         assert len(header) == 1
-        assert header.keys() == ['E']
+        assert list(header) == ['E']
 
         default = header.pop('X', 'Y')
         assert default == 'Y'
@@ -1117,7 +1142,7 @@ class TestHeaderFunctions(FitsTestCase):
         # operation when the header is coming from another HDU
         hdu.header.extend(hdu2.header, strip=False)
         assert len(hdu.header) == 11
-        assert hdu.header.keys()[5] == 'XTENSION'
+        assert list(hdu.header)[5] == 'XTENSION'
         assert hdu.header[-1] == 'some val'
         assert ('MYKEY', 1) in hdu.header
 
@@ -1230,7 +1255,7 @@ class TestHeaderFunctions(FitsTestCase):
         header = fits.Header([('A', 'B'), ('C', 'D')])
         header.append('E')
         assert len(header) == 3
-        assert header.keys()[-1] == 'E'
+        assert list(header)[-1] == 'E'
         assert header[-1] == ''
         assert header.comments['E'] == ''
 
@@ -1240,7 +1265,7 @@ class TestHeaderFunctions(FitsTestCase):
         header.append('')
         assert len(header) == 4
 
-        assert header.keys()[-1] == ''
+        assert list(header)[-1] == ''
         assert header[''] == ''
         assert header.comments[''] == ''
 
@@ -1301,6 +1326,10 @@ class TestHeaderFunctions(FitsTestCase):
         assert list(header.keys())[-1] == 'TEST2'
         assert list(header.keys())[-3] == 'TEST1'
 
+    def test_remove(self):
+        # TODO: Test the Header.remove() method; add support for ignore_missing
+        pass
+
     def test_header_comments(self):
         header = fits.Header([('A', 'B', 'C'), ('DEF', 'G', 'H')])
         assert (repr(header.comments) ==
@@ -1347,7 +1376,7 @@ class TestHeaderFunctions(FitsTestCase):
     def test_commentary_slicing(self):
         header = fits.Header()
 
-        indices = range(5)
+        indices = list(range(5))
 
         for idx in indices:
             header['HISTORY'] = idx
@@ -1423,7 +1452,7 @@ class TestHeaderFunctions(FitsTestCase):
         header['HISTORY'] = longval
 
         assert len(header) == 7
-        assert header.keys()[2] == 'FRED'
+        assert list(header)[2] == 'FRED'
         assert str(header.cards[3]) == 'HISTORY ' + longval[:72]
         assert str(header.cards[4]).rstrip() == 'HISTORY ' + longval[72:]
 
@@ -1433,7 +1462,7 @@ class TestHeaderFunctions(FitsTestCase):
         assert str(header.cards[2]).rstrip() == 'HISTORY ' + longval[72:]
 
     def test_header_fromtextfile(self):
-        """Regression test for https://trac.assembla.com/pyfits/ticket/122
+        """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/122
 
         Manually write a text file containing some header cards ending with
         newlines and ensure that fromtextfile can read them back in.
@@ -1451,7 +1480,7 @@ class TestHeaderFunctions(FitsTestCase):
         assert header == header2
 
     def test_header_fromtextfile_with_end_card(self):
-        """Regression test for https://trac.assembla.com/pyfits/ticket/154
+        """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/154
 
         Make sure that when a Header is read from a text file that the END card
         is ignored.
@@ -1474,7 +1503,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_append_end_card(self):
         """
-        Regression test 2 for https://trac.assembla.com/pyfits/ticket/154
+        Regression test 2 for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/154
 
         Manually adding an END card to a header should simply result in a
         ValueError (as was the case in PyFITS 3.0 and earlier).
@@ -1493,7 +1522,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_invalid_end_cards(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/217
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/217
 
         This tests the case where the END card looks like a normal card like
         'END = ' and other similar oddities.  As long as a card starts with END
@@ -1566,7 +1595,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_unnecessary_move(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/125
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/125
 
         Ensures that a header is not modified when setting the position of a
         keyword that's already in its correct position.
@@ -1575,37 +1604,37 @@ class TestHeaderFunctions(FitsTestCase):
         header = fits.Header([('A', 'B'), ('B', 'C'), ('C', 'D')])
 
         header.set('B', before=2)
-        assert header.keys() == ['A', 'B', 'C']
+        assert list(header) == ['A', 'B', 'C']
         assert not header._modified
 
         header.set('B', after=0)
-        assert header.keys() == ['A', 'B', 'C']
+        assert list(header) == ['A', 'B', 'C']
         assert not header._modified
 
         header.set('B', before='C')
-        assert header.keys() == ['A', 'B', 'C']
+        assert list(header) == ['A', 'B', 'C']
         assert not header._modified
 
         header.set('B', after='A')
-        assert header.keys() == ['A', 'B', 'C']
+        assert list(header) == ['A', 'B', 'C']
         assert not header._modified
 
         header.set('B', before=2)
-        assert header.keys() == ['A', 'B', 'C']
+        assert list(header) == ['A', 'B', 'C']
         assert not header._modified
 
         # 123 is well past the end, and C is already at the end, so it's in the
         # right place already
         header.set('C', before=123)
-        assert header.keys() == ['A', 'B', 'C']
+        assert list(header) == ['A', 'B', 'C']
         assert not header._modified
 
         header.set('C', after=123)
-        assert header.keys() == ['A', 'B', 'C']
+        assert list(header) == ['A', 'B', 'C']
         assert not header._modified
 
     def test_invalid_float_cards(self):
-        """Regression test for https://trac.assembla.com/pyfits/ticket/137"""
+        """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/137"""
 
         # Create a header containing two of the problematic cards in the test
         # case where this came up:
@@ -1652,7 +1681,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_invalid_float_cards2(self, capsys):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/140
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/140
         """
 
         # The example for this test requires creating a FITS file containing a
@@ -1676,13 +1705,13 @@ class TestHeaderFunctions(FitsTestCase):
         # should be output as separate warning messages, but that's
         # something to think about...
         msg = str(w[3].message)
-        assert '(invalid value string: 5.0022221e-07)' in msg
+        assert "(invalid value string: '5.0022221e-07')" in msg
 
     def test_leading_zeros(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/137, part 2
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/137, part 2
 
-        Ticket https://trac.assembla.com/pyfits/ticket/137 also showed that in
+        Ticket https://aeon.stsci.edu/ssb/trac/pyfits/ticket/137 also showed that in
         float values like 0.001 the leading zero was unnecessarily being
         stripped off when rewriting the header.  Though leading zeros should be
         removed from integer values to prevent misinterpretation as octal by
@@ -1702,7 +1731,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_assign_boolean(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/123
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/123
 
         Tests assigning Python and Numpy boolean values to keyword values.
         """
@@ -1735,7 +1764,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_header_method_keyword_normalization(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/149
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/149
 
         Basically ensures that all public Header methods are case-insensitive
         w.r.t. keywords.
@@ -1744,7 +1773,7 @@ class TestHeaderFunctions(FitsTestCase):
         """
 
         h = fits.Header([('abC', 1), ('Def', 2), ('GeH', 3)])
-        assert h.keys() == ['ABC', 'DEF', 'GEH']
+        assert list(h) == ['ABC', 'DEF', 'GEH']
         assert 'abc' in h
         assert 'dEf' in h
 
@@ -1760,14 +1789,14 @@ class TestHeaderFunctions(FitsTestCase):
         assert len(h) == 3
 
         del h['gEh']
-        assert h.keys() == ['ABC', 'DEF']
+        assert list(h) == ['ABC', 'DEF']
         assert len(h) == 2
         assert h.get('def') == 2
 
         h.set('Abc', 3)
         assert h['ABC'] == 3
         h.set('gEh', 3, before='Abc')
-        assert h.keys() == ['GEH', 'ABC', 'DEF']
+        assert list(h) == ['GEH', 'ABC', 'DEF']
 
         assert h.pop('abC') == 3
         assert len(h) == 2
@@ -1776,11 +1805,11 @@ class TestHeaderFunctions(FitsTestCase):
         assert len(h) == 2
         assert h.setdefault('aBc', 1) == 1
         assert len(h) == 3
-        assert h.keys() == ['GEH', 'DEF', 'ABC']
+        assert list(h) == ['GEH', 'DEF', 'ABC']
 
         h.update({'GeH': 1, 'iJk': 4})
         assert len(h) == 4
-        assert h.keys() == ['GEH', 'DEF', 'ABC', 'IJK']
+        assert list(h) == ['GEH', 'DEF', 'ABC', 'IJK']
         assert h['GEH'] == 1
 
         assert h.count('ijk') == 1
@@ -1788,11 +1817,11 @@ class TestHeaderFunctions(FitsTestCase):
 
         h.remove('Def')
         assert len(h) == 3
-        assert h.keys() == ['GEH', 'ABC', 'IJK']
+        assert list(h) == ['GEH', 'ABC', 'IJK']
 
     def test_end_in_comment(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/142
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/142
 
         Tests a case where the comment of a card ends with END, and is followed
         by several blank cards.
@@ -1832,7 +1861,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_assign_unicode(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/134
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/134
 
         Assigning a unicode literal as a header value should not fail silently.
         If the value can be converted to ASCII then it should just work.
@@ -1841,37 +1870,37 @@ class TestHeaderFunctions(FitsTestCase):
         Also tests unicode for keywords and comments.
         """
 
-        erikku = u'\u30a8\u30ea\u30c3\u30af'
+        erikku = u('\u30a8\u30ea\u30c3\u30af')
 
         def assign(keyword, val):
             h[keyword] = val
 
         h = fits.Header()
-        h[u'FOO'] = 'BAR'
+        h[u('FOO')] = 'BAR'
         assert 'FOO' in h
         assert h['FOO'] == 'BAR'
-        assert h[u'FOO'] == 'BAR'
+        assert h[u('FOO')] == 'BAR'
         assert repr(h) == _pad("FOO     = 'BAR     '")
         pytest.raises(ValueError, assign, erikku, 'BAR')
 
-        h['FOO'] = u'BAZ'
-        assert h[u'FOO'] == 'BAZ'
-        assert h[u'FOO'] == u'BAZ'
+        h['FOO'] = u('BAZ')
+        assert h[u('FOO')] == 'BAZ'
+        assert h[u('FOO')] == u('BAZ')
         assert repr(h) == _pad("FOO     = 'BAZ     '")
         pytest.raises(ValueError, assign, 'FOO', erikku)
 
-        h['FOO'] = ('BAR', u'BAZ')
+        h['FOO'] = ('BAR', u('BAZ'))
         assert h['FOO'] == 'BAR'
-        assert h['FOO'] == u'BAR'
+        assert h['FOO'] == u('BAR')
         assert h.comments['FOO'] == 'BAZ'
-        assert h.comments['FOO'] == u'BAZ'
+        assert h.comments['FOO'] == u('BAZ')
         assert repr(h) == _pad("FOO     = 'BAR     '           / BAZ")
 
-        h['FOO'] = (u'BAR', u'BAZ')
+        h['FOO'] = (u('BAR'), u('BAZ'))
         assert h['FOO'] == 'BAR'
-        assert h['FOO'] == u'BAR'
+        assert h['FOO'] == u('BAR')
         assert h.comments['FOO'] == 'BAZ'
-        assert h.comments['FOO'] == u'BAZ'
+        assert h.comments['FOO'] == u('BAZ')
         assert repr(h) == _pad("FOO     = 'BAR     '           / BAZ")
 
         pytest.raises(ValueError, assign, 'FOO', ('BAR', erikku))
@@ -1894,20 +1923,21 @@ class TestHeaderFunctions(FitsTestCase):
         """
 
         h = fits.Header()
-        if PY3:
+        if six.PY3:
             pytest.raises(ValueError, h.set, 'TEST',
                           bytes('Hello', encoding='ascii'))
-        else:
+        elif six.PY2:
             pytest.raises(ValueError, h.set, 'TEST', str('ñ'))
 
     def test_header_strip_whitespace(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/146, and
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/146, and
         for the solution that is optional stripping of whitespace from the end
         of a header value.
 
         By default extra whitespace is stripped off, but if
-        fits.STRIP_HEADER_WHITESPACE = False it should not be stripped.
+        `fits.conf.strip_header_whitespace` = False it should not be
+        stripped.
         """
 
         h = fits.Header()
@@ -1919,14 +1949,11 @@ class TestHeaderFunctions(FitsTestCase):
         assert h.cards['FOO'].image.rstrip() == "FOO     = 'Bar      '"
         assert h.cards['QUX'].image.rstrip() == "QUX     = 'Bar        '"
 
-        fits.STRIP_HEADER_WHITESPACE.set(False)
-        try:
+        with fits.conf.set_temp('strip_header_whitespace', False):
             assert h['FOO'] == 'Bar      '
             assert h['QUX'] == 'Bar        '
             assert h.cards['FOO'].image.rstrip() == "FOO     = 'Bar      '"
             assert h.cards['QUX'].image.rstrip() == "QUX     = 'Bar        '"
-        finally:
-            fits.STRIP_HEADER_WHITESPACE.set(True)
 
         assert h['FOO'] == 'Bar'
         assert h['QUX'] == 'Bar'
@@ -1935,7 +1962,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_keep_duplicate_history_in_orig_header(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/156
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/156
 
         When creating a new HDU from an existing Header read from an existing
         FITS file, if the origianl header contains duplicate HISTORY values
@@ -1968,7 +1995,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_invalid_keyword_cards(self):
         """
-        Test for https://trac.assembla.com/pyfits/ticket/109
+        Test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/109
 
         Allow opening files with headers containing invalid keywords.
         """
@@ -2010,7 +2037,7 @@ class TestHeaderFunctions(FitsTestCase):
 
     def test_fix_hierarch_with_invalid_value(self, capsys):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/172
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/172
 
         Ensures that when fixing a hierarch card it remains a hierarch card.
         """
@@ -2310,7 +2337,7 @@ class TestRecordValuedKeywordCards(FitsTestCase):
         string of the first card with that keyword.
 
         This test was changed to reflect the requirement in ticket
-        https://trac.assembla.com/pyfits/ticket/184--previously it required
+        https://aeon.stsci.edu/ssb/trac/pyfits/ticket/184--previously it required
         _test_header['DP1'] to return the parsed float value.
         """
 
@@ -2391,10 +2418,9 @@ class TestRecordValuedKeywordCards(FitsTestCase):
 
         del self._test_header['DP1.AXIS.1']
         assert len(self._test_header) == 7
-        assert self._test_header.keys()[0] == 'DP1.NAXIS'
+        assert list(self._test_header)[0] == 'DP1.NAXIS'
         assert self._test_header[0] == 2
-        assert self._test_header.keys()[1] == 'DP1.AXIS.2'
-        assert self._test_header[1] == 2
+        assert list(self._test_header)[1] == 'DP1.AXIS.2'
 
         # Perform a subsequent delete to make sure all the index mappings were
         # updated
@@ -2440,9 +2466,9 @@ class TestRecordValuedKeywordCards(FitsTestCase):
 
         del self._test_header['DP1.A*...']
         assert len(self._test_header) == 2
-        assert self._test_header.keys()[0] == 'DP1.NAXIS'
+        assert list(self._test_header)[0] == 'DP1.NAXIS'
         assert self._test_header[0] == 2
-        assert self._test_header.keys()[1] == 'DP1.NAUX'
+        assert list(self._test_header)[1] == 'DP1.NAUX'
         assert self._test_header[1] == 2
 
     def test_successive_pattern_matching(self):
@@ -2473,7 +2499,7 @@ class TestRecordValuedKeywordCards(FitsTestCase):
         """
 
         cl = self._test_header['DP1.AXIS.*']
-        assert cl.keys() == ['DP1.AXIS.1', 'DP1.AXIS.2']
+        assert list(cl) == ['DP1.AXIS.1', 'DP1.AXIS.2']
 
     def test_rvkc_in_cardlist_values(self):
         """
@@ -2482,7 +2508,7 @@ class TestRecordValuedKeywordCards(FitsTestCase):
         """
 
         cl = self._test_header['DP1.AXIS.*']
-        assert cl.values() == [1.0, 2.0]
+        assert list(cl.values()) == [1.0, 2.0]
 
     def test_rvkc_value_attribute(self):
         """
@@ -2496,7 +2522,7 @@ class TestRecordValuedKeywordCards(FitsTestCase):
 
     def test_overly_permissive_parsing(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/183
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/183
 
         Ensures that cards with standard commentary keywords are never treated
         as RVKCs.  Also ensures that cards not stricly matching the RVKC
@@ -2533,7 +2559,7 @@ class TestRecordValuedKeywordCards(FitsTestCase):
 
     def test_overly_aggressive_rvkc_lookup(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/184
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/184
 
         Ensures that looking up a RVKC by keyword only (without the
         field-specifier) in a header returns the full string value of that card
@@ -2555,3 +2581,22 @@ class TestRecordValuedKeywordCards(FitsTestCase):
         pytest.raises(KeyError, lambda: h['FOO.AXIS'])
         pytest.raises(KeyError, lambda: h['FOO.AXIS.'])
         pytest.raises(KeyError, lambda: h['FOO.'])
+
+    def test_fitsheader_script(self):
+        """
+        Checks the basic functionality of the fitsheader script
+        """
+        from ....io.fits.scripts import fitsheader
+        # Can an extension by specified by the EXTNAME keyword?
+        hf = fitsheader.HeaderFormatter(self.data('zerowidth.fits'))
+        assert "EXTNAME = 'AIPS FQ" in hf.parse('AIPS FQ')
+
+        # Can an extension by specified by the EXTNAME+EXTVER keywords?
+        hf = fitsheader.HeaderFormatter(self.data('test0.fits'))
+        assert "EXTNAME = 'SCI" in hf.parse('SCI,2')
+        # fitsheader should only print the compressed header when asked
+        hf = fitsheader.HeaderFormatter(self.data('comp.fits'))
+        assert "XTENSION= 'IMAGE" in hf.parse(1)  # decompressed
+        hf = fitsheader.HeaderFormatter(self.data('comp.fits'),
+                                        compressed=True)
+        assert "XTENSION= 'BINTABLE" in hf.parse(1)  # compressed
