@@ -5,13 +5,15 @@ not meant to be used directly, but instead are available as readers/writers in
 `astropy.table`. See :ref:`table_io` for more details.
 """
 
-from __future__ import print_function
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
 import os
 import warnings
 
 import numpy as np
 from ...utils.exceptions import AstropyUserWarning
+from ...extern import six
 
 HDF5_SIGNATURE = b'\x89HDF\r\n\x1a\n'
 
@@ -56,15 +58,16 @@ def read_table_hdf5(input, path=None):
     """
     Read a Table object from an HDF5 file
 
-    This requires `h5py <http://alfven.org/wp/hdf5-for-python/>`_ to be
-    installed. If more than one table is present in the HDF5 file or group, the
-    first table is read in and a warning is displayed.
+    This requires `h5py <http://h5py.org/>`_ to be installed. If more than one
+    table is present in the HDF5 file or group, the first table is read in and
+    a warning is displayed.
 
     Parameters
     ----------
-    input : str or `h5py.highlevel.File` or `h5py.highlevel.Group` or `h5py.highlevel.Dataset`
-        If a string, the filename to read the table from. If an h5py object,
-        either the file or the group object to read the table from.
+    input : str or :class:`h5py:File` or :class:`h5py:Group` or
+        :class:`h5py:Dataset` If a string, the filename to read the table from.
+        If an h5py object, either the file or the group object to read the
+        table from.
     path : str
         The path from which to read the table inside the HDF5 file.
         This should be relative to the input file or group.
@@ -96,12 +99,14 @@ def read_table_hdf5(input, path=None):
             arrays = _find_all_structured_arrays(input)
 
             if len(arrays) == 0:
-                raise ValueError("no table found in HDF5 group {0}".format(path))
+                raise ValueError("no table found in HDF5 group {0}".
+                                 format(path))
             elif len(arrays) > 0:
                 path = arrays[0] if path is None else path + '/' + arrays[0]
                 warnings.warn("path= was not specified but multiple tables"
                               " are present, reading in first available"
-                              " table (path={0})".format(path), AstropyUserWarning)
+                              " table (path={0})".format(path),
+                              AstropyUserWarning)
                 return read_table_hdf5(input, path=path)
 
     elif not isinstance(input, h5py.highlevel.Dataset):
@@ -143,27 +148,30 @@ def write_table_hdf5(table, output, path=None, compression=False,
     """
     Write a Table object to an HDF5 file
 
-    This requires `h5py <http://alfven.org/wp/hdf5-for-python/>`_ to be
-    installed.
+    This requires `h5py <http://h5py.org/>`_ to be installed.
 
     Parameters
     ----------
-    output : str or `h5py.highlevel.File` or `h5py.highlevel.Group`
+    table : `~astropy.table.Table`
+        Data table that is to be written to file.
+    output : str or :class:`h5py:File` or :class:`h5py:Group`
         If a string, the filename to write the table to. If an h5py object,
         either the file or the group object to write the table to.
+    path : str
+        The path to which to write the table inside the HDF5 file.
+        This should be relative to the input file or group.
     compression : bool or str or int
         Whether to compress the table inside the HDF5 file. If set to `True`,
         ``'gzip'`` compression is used. If a string is specified, it should be
         one of ``'gzip'``, ``'szip'``, or ``'lzf'``. If an integer is
         specified (in the range 0-9), ``'gzip'`` compression is used, and the
         integer denotes the compression level.
-    path : str
-        The path to which to write the table inside the HDF5 file.
-        This should be relative to the input file or group.
     append : bool
         Whether to append the table to an existing HDF5 file.
     overwrite : bool
         Whether to overwrite any existing file without warning.
+        If ``append=True`` and ``overwrite=True`` then only the dataset will be
+        replaced; the file/group will not be overwritten.
     """
 
     try:
@@ -191,10 +199,10 @@ def write_table_hdf5(table, output, path=None, compression=False,
         else:
             output_group = output
 
-    elif isinstance(output, basestring):
+    elif isinstance(output, six.string_types):
 
         if os.path.exists(output) and not append:
-            if overwrite:
+            if overwrite and not append:
                 os.remove(output)
             else:
                 raise IOError("File exists: {0}".format(output))
@@ -212,17 +220,23 @@ def write_table_hdf5(table, output, path=None, compression=False,
 
     else:
 
-        raise TypeError('output should be a string or an h5py File or Group object')
+        raise TypeError('output should be a string or an h5py File or '
+                        'Group object')
 
     # Check whether table already exists
     if name in output_group:
-        raise IOError("Table {0} already exists".format(path))
+        if append and overwrite:
+            # Delete only the dataset itself
+            del output_group[name]
+        else:
+            raise IOError("Table {0} already exists".format(path))
 
     # Write the table to the file
     if compression:
         if compression is True:
             compression = 'gzip'
-        dset = output_group.create_dataset(name, data=table._data, compression=compression)
+        dset = output_group.create_dataset(name, data=table._data,
+                                           compression=compression)
     else:
         dset = output_group.create_dataset(name, data=table._data)
 
@@ -233,4 +247,5 @@ def write_table_hdf5(table, output, path=None, compression=False,
             dset.attrs[key] = val
         except TypeError:
             warnings.warn("Attribute `{0}` of type {1} cannot be written to "
-                          "HDF5 files - skipping".format(key, type(val)), AstropyUserWarning)
+                          "HDF5 files - skipping".format(key, type(val)),
+                          AstropyUserWarning)
