@@ -4,9 +4,8 @@ from __future__ import print_function
 
 import numpy as np
 
-from ....tests.helper import pytest
+from ....tests.helper import pytest, catch_warnings
 from ....table import Table, Column
-from .... import log
 
 try:
     import h5py
@@ -190,6 +189,25 @@ def test_read_write_existing_append_groups(tmpdir):
     assert np.all(t3['a'] == [1, 2, 3])
 
 
+@pytest.mark.skipif('not HAS_H5PY')
+def test_read_write_existing_append_overwrite(tmpdir):
+    test_file = str(tmpdir.join('test.hdf5'))
+    t1 = Table()
+    t1.add_column(Column(name='a', data=[1, 2, 3]))
+    t1.write(test_file, path='table1')
+    t1.write(test_file, path='table2', append=True)
+    t1v2 = Table()
+    t1v2.add_column(Column(name='a', data=[4, 5, 6]))
+    with pytest.raises(IOError) as exc:
+        t1v2.write(test_file, path='table1', append=True)
+    assert exc.value.args[0] == 'Table table1 already exists'
+    t1v2.write(test_file, path='table1', append=True, overwrite=True)
+    t2 = Table.read(test_file, path='table1')
+    assert np.all(t2['a'] == [4, 5, 6])
+    t3 = Table.read(test_file, path='table2')
+    assert np.all(t3['a'] == [1, 2, 3])
+
+
 @pytest.mark.skipif('not HAS_H5PY')
 def test_read_fileobj(tmpdir):
 
@@ -366,6 +384,8 @@ def test_skip_meta(tmpdir):
     t1.meta['e'] = np.array([1, 2, 3])
     t1.meta['f'] = str
 
-    with log.log_to_list() as warning_list:
+    with catch_warnings() as w:
         t1.write(test_file, path='the_table')
-    assert warning_list[0].message.startswith("Attribute `f` of type {0} cannot be written to HDF5 files - skipping".format(type(t1.meta['f'])))
+    assert len(w) == 1
+    assert str(w[0].message).startswith(
+        "Attribute `f` of type {0} cannot be written to HDF5 files - skipping".format(type(t1.meta['f'])))
