@@ -1,5 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+# TEST_UNICODE_LITERALS
+
 import operator
 
 from distutils import version
@@ -14,13 +16,6 @@ from ... import units as u
 NUMPY_LT_1P8 = [int(x) for x in np.__version__.split('.')[:2]] < [1, 8]
 
 
-@pytest.fixture(params=[table.Column, table.MaskedColumn])
-def Column(request):
-    # Fixture to run all the Column tests for both an unmasked (ndarray)
-    # and masked (MaskedArray) column.
-    return request.param
-
-
 class TestColumn():
 
     def test_1(self, Column):
@@ -71,13 +66,12 @@ class TestColumn():
 
     def test_format(self, Column):
         """Show that the formatted output from str() works"""
-        MAX_LINES_val = table.pprint.MAX_LINES()
-        table.pprint.MAX_LINES.set(7)
-        c1 = Column(np.arange(2000), name='a', dtype=float,
-                    format='%6.2f')
-        assert str(c1) == ('   a   \n-------\n   0.00\n'
-                           '   1.00\n    ...\n1998.00\n1999.00')
-        table.pprint.MAX_LINES.set(MAX_LINES_val)
+        from ... import conf
+        with conf.set_temp('max_lines', 7):
+            c1 = Column(np.arange(2000), name='a', dtype=float,
+                        format='%6.2f')
+            assert str(c1) == ('   a   \n-------\n   0.00\n'
+                               '   1.00\n    ...\n1998.00\n1999.00')
 
     def test_convert_numpy_array(self, Column):
         d = Column([1, 2, 3], name='a', dtype='i8')
@@ -94,21 +88,6 @@ class TestColumn():
         d.convert_unit_to("km")
         assert np.all(d.data == [0.001, 0.002, 0.003])
 
-    def test_deprecated_attributes(self, Column, recwarn):
-        d = Column([1, 2, 3], name='a', dtype="f8", unit="m")
-
-        with catch_warnings(AstropyDeprecationWarning) as warning_lines:
-            d.units
-            assert warning_lines[0].category == AstropyDeprecationWarning
-
-        with catch_warnings(AstropyDeprecationWarning) as warning_lines:
-            c = Column([1,2,3], name='a', dtypes="f8", unit="m")
-            assert warning_lines[0].category == AstropyDeprecationWarning
-
-        with catch_warnings(AstropyDeprecationWarning) as warning_lines:
-            c = Column([1,2,3], name='a', dtype="f8", units="m")
-            assert warning_lines[0].category == AstropyDeprecationWarning
-
     def test_array_wrap(self):
         """Test that the __array_wrap__ method converts a reduction ufunc
         output that has a different shape into an ndarray view.  Without this a
