@@ -1,9 +1,13 @@
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
 import sys
 
-from ...tests.helper import pytest
 from ...utils.data import get_pkg_data_filename
 
 fd = None
+
+
 def test_open_file_detection():
     global fd
     fd = open(get_pkg_data_filename('data/open_file_detection.txt'))
