@@ -7,18 +7,25 @@ A collection of different unit formats.
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+from inspect import isclass
+
 from .base import Base
 from .generic import Generic, Unscaled
 from .cds import CDS
 from .console import Console
 from .fits import Fits
 from .latex import Latex
+from .ogip import OGIP
 from .unicode_format import Unicode
 from .vounit import VOUnit
 
 __all__ = [
-    'Generic', 'CDS', 'Console', 'Fits', 'Latex', 'Unicode', 'Unscaled',
-    'VOUnit', 'get_format']
+    'Base', 'Generic', 'CDS', 'Console', 'Fits', 'Latex', 'OGIP', 'Unicode',
+    'Unscaled', 'VOUnit', 'get_format']
+
+FORMATS = [
+    'Generic', 'CDS', 'Console', 'Fits', 'Latex', 'OGIP', 'Unicode',
+    'Unscaled', 'VOUnit']
 
 
 def get_format(format=None):
@@ -44,8 +51,8 @@ def get_format(format=None):
     if format is None:
         format = 'generic'
     format = format.lower()
-    for key in __all__:
+    for key in FORMATS:
         val = globals()[key]
-        if (issubclass(val, Base) and key.lower() == format.lower()):
+        if isclass(val) and (issubclass(val, Base) and key.lower() == format.lower()):
             return val()
     raise ValueError("Unknown format {0!r}".format(format))
