@@ -12,6 +12,9 @@ Includes the following fixes:
   alias to `inspect.getmodule` if the stdlib version is correct, but for
   versions of python with the bug, it uses an internal patched version.
 
+* The `contextlib.ignored` context manager, which is only available in Python
+  3.4 or greater.
+
 """
 
 from __future__ import (absolute_import, division, print_function,
@@ -25,7 +28,7 @@ from functools import wraps
 
 
 __all__ = ['inspect_getmodule', 'invalidate_caches', 'override__dir__',
-           'possible_filename']
+           'ignored', 'possible_filename']
 
 
 def possible_filename(filename):
@@ -164,3 +167,30 @@ def override__dir__(f):
             return sorted(members)
 
     return override__dir__wrapper
+
+
+try:
+    from contextlib import ignored
+except ImportError:
+    from contextlib import contextmanager
+    @contextmanager
+    def ignored(*exceptions):
+        """A context manager for ignoring exceptions.  Equivalent to::
+
+            try:
+                <body>
+            except exceptions:
+                pass
+
+        Example::
+
+            >>> import os
+            >>> with ignored(OSError):
+            ...     os.remove('file-that-does-not-exist')
+
+        """
+
+        try:
+            yield
+        except exceptions:
+            pass
