@@ -377,7 +377,7 @@ PyWcsprm_init(
 
     wcsvfree(&nwcs, &wcs);
 
-    if (convert_rejections_to_warnings(wcsprintf_buf())) {
+    if (convert_rejections_to_warnings()) {
       free(colsel_ints);
       return -1;
     }
@@ -452,6 +452,37 @@ PyWcsprm_init(
   }
 }
 
+/*@null@*/ static PyObject*
+PyWcsprm_bounds_check(
+    PyWcsprm* self,
+    PyObject* args,
+    PyObject* kwds) {
+
+  unsigned char pix2sky    = 1;
+  unsigned char sky2pix    = 1;
+  int           bounds     = 0;
+  const char*   keywords[] = {"pix2world", "world2pix", NULL};
+
+  if (!PyArg_ParseTupleAndKeywords(
+          args, kwds, "|bb:bounds_check", (char **)keywords,
+          &pix2sky, &sky2pix)) {
+    return NULL;
+  }
+
+  if (pix2sky) {
+      bounds |= 2|4;
+  }
+
+  if (sky2pix) {
+      bounds |= 1;
+  }
+
+  wcsbchk(&self->x, bounds);
+
+  Py_RETURN_NONE;
+}
+
+
 /*@null@*/ static PyObject*
 PyWcsprm_copy(
     PyWcsprm* self) {
@@ -464,6 +495,8 @@ PyWcsprm_copy(
     return NULL;
   }
 
+  wcsini(0, self->x.naxis, &copy->x);
+
   wcsprm_python2c(&self->x);
   status = wcscopy(1, &self->x, &copy->x);
   wcsprm_c2python(&self->x);
@@ -585,7 +618,7 @@ PyWcsprm_find_all_wcs(
 
   wcsvfree(&nwcs, &wcs);
 
-  if (convert_rejections_to_warnings(wcsprintf_buf())) {
+  if (convert_rejections_to_warnings()) {
     return NULL;
   }
 
@@ -696,6 +729,44 @@ PyWcsprm_celfix(
   }
 }
 
+static PyObject *
+PyWcsprm_compare(
+    PyWcsprm* self,
+    PyObject* args,
+    PyObject* kwds) {
+
+  int cmp = 0;
+  PyWcsprm *other;
+  int equal;
+  int status;
+
+  const char* keywords[] = {"other", "cmp", NULL};
+
+  if (!PyArg_ParseTupleAndKeywords(
+          args, kwds, "O!|i:compare", (char **)keywords,
+          &PyWcsprmType, &other, &cmp)) {
+    return NULL;
+  }
+
+
+  wcsprm_python2c(&self->x);
+  wcsprm_python2c(&other->x);
+  status = wcscompare(cmp, &self->x, &other->x, &equal);
+  wcsprm_c2python(&self->x);
+  wcsprm_c2python(&other->x);
+
+  if (status) {
+    wcserr_fix_to_python_exc(self->x.err);
+    return NULL;
+  } else {
+    if (equal) {
+      Py_RETURN_TRUE;
+    } else {
+      Py_RETURN_FALSE;
+    }
+  }
+}
+
 /*@null@*/ static PyObject*
 PyWcsprm_cylfix(
     PyWcsprm* self,
@@ -1611,6 +1682,45 @@ PyWcsprm___str__(
   #endif
 }
 
+PyObject *PyWcsprm_richcompare(PyObject *a, PyObject *b, int op) {
+  int equal;
+  int status;
+
+  struct wcsprm *ax;
+  struct wcsprm *bx;
+
+  if ((op == Py_EQ || op == Py_NE) &&
+      PyObject_TypeCheck(b, &PyWcsprmType)) {
+    ax = &((PyWcsprm *)a)->x;
+    bx = &((PyWcsprm *)b)->x;
+
+    wcsprm_python2c(ax);
+    wcsprm_python2c(bx);
+    status = wcscompare(
+        WCSCOMPARE_ANCILLARY,
+        ax, bx, &equal);
+    wcsprm_c2python(ax);
+    wcsprm_c2python(bx);
+
+    if (status == 0) {
+      if (op == Py_NE) {
+        equal = !equal;
+      }
+      if (equal) {
+        Py_RETURN_TRUE;
+      } else {
+        Py_RETURN_FALSE;
+      }
+    } else {
+      wcs_to_python_exc(&(((PyWcsprm *)a)->x));
+      return NULL;
+    }
+  }
+
+  Py_INCREF(Py_NotImplemented);
+  return Py_NotImplemented;
+}
+
 /*@null@*/ static PyObject*
 PyWcsprm_sub(
     PyWcsprm* self,
@@ -1628,7 +1738,6 @@ PyWcsprm_sub(
   char*      element_str    = NULL;
   int        element_val    = 0;
   int        nsub           = 0;
-  int        alloc_size     = 0;
   int*       axes           = NULL;
   int        status         = -1;
   const char*    keywords[] = {"axes", NULL};
@@ -1641,7 +1750,7 @@ PyWcsprm_sub(
 
   if (py_axes == NULL || py_axes == Py_None) {
     /* leave all variables as is */
-  } else if (PySequence_Check(py_axes)) {
+  } else if (PyList_Check(py_axes) || PyTuple_Check(py_axes)) {
     tmp = PySequence_Size(py_axes);
     if (tmp == -1) {
       goto exit;
@@ -1740,11 +1849,9 @@ PyWcsprm_sub(
     goto exit;
   }
 
-  alloc_size = self->x.naxis;
-
   py_dest_wcs = (PyWcsprm*)PyWcsprm_cnew();
   py_dest_wcs->x.flag = -1;
-  status = wcsini(0, alloc_size, &py_dest_wcs->x);
+  status = wcsini(0, nsub, &py_dest_wcs->x);
   if (status != 0) {
     goto exit;
   }
@@ -3245,8 +3352,10 @@ static PyGetSetDef PyWcsprm_getset[] = {
 };
 
 static PyMethodDef PyWcsprm_methods[] = {
+  {"bounds_check", (PyCFunction)PyWcsprm_bounds_check, METH_VARARGS|METH_KEYWORDS, doc_bounds_check},
   {"cdfix", (PyCFunction)PyWcsprm_cdfix, METH_NOARGS, doc_cdfix},
   {"celfix", (PyCFunction)PyWcsprm_celfix, METH_NOARGS, doc_celfix},
+  {"compare", (PyCFunction)PyWcsprm_compare, METH_VARARGS|METH_KEYWORDS, doc_compare},
   {"__copy__", (PyCFunction)PyWcsprm_copy, METH_NOARGS, doc_copy},
   {"cylfix", (PyCFunction)PyWcsprm_cylfix, METH_VARARGS|METH_KEYWORDS, doc_cylfix},
   {"datfix", (PyCFunction)PyWcsprm_datfix, METH_NOARGS, doc_datfix},
@@ -3307,7 +3416,7 @@ PyTypeObject PyWcsprmType = {
   doc_Wcsprm,                   /* tp_doc */
   0,                            /* tp_traverse */
   0,                            /* tp_clear */
-  0,                            /* tp_richcompare */
+  PyWcsprm_richcompare,         /* tp_richcompare */
   0,                            /* tp_weaklistoffset */
   0,                            /* tp_iter */
   0,                            /* tp_iternext */
@@ -3373,5 +3482,8 @@ _setup_wcsprm_type(
     CONSTANT(WCSHDO_PVn_ma)    ||
     CONSTANT(WCSHDO_CRPXna)    ||
     CONSTANT(WCSHDO_CNAMna)    ||
-    CONSTANT(WCSHDO_WCSNna));
+    CONSTANT(WCSHDO_WCSNna)    ||
+    CONSTANT(WCSCOMPARE_ANCILLARY) ||
+    CONSTANT(WCSCOMPARE_TILING) ||
+    CONSTANT(WCSCOMPARE_CRPIX));
 }
